"""Plugin Base class"""

# flake8: noqa: E402
# -*- coding: utf-8 -*-
# (c) 2025 BeyondTrust Inc.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
import logging

from ansible.errors import AnsibleLookupError
from ansible.plugins.lookup import LookupBase

from secrets_safe_library.integrations.ansible.common_utils.logger import Logger


class BaseLookup(LookupBase):
    """
    Plugin Base class.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.auth_data = {}
        self.log = Logger(
            kwargs.get("log_level", logging.DEBUG),
            kwargs.get("log_file_name", "ansible-collection.log"),
        )

    def validate_and_set_auth_parameters(self, params):
        """
        Validate required parameters and set them in the auth_data dictionary.
        Args:
            params (dict): Parameters to validate.
        """

        required_params = ["client_id", "client_secret", "api_url"]

        missing = [p for p in required_params if not params.get(p)]
        if missing:
            raise AnsibleLookupError(
                f"Missing required parameter(s): {', '.join(missing)}"
            )

        self.auth_data.update(
            {
                "client_id": params.get("client_id"),
                "client_secret": params.get("client_secret"),
                "api_url": params.get("api_url"),
                "api_version": params.get("api_version"),
            }
        )



"""Databases module, all the logic to manage databases from BeyondInsight API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_DATABASES_ASSET_ID,
    PUT_DATABASES_ID,
)
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.databases import fields as databases_fields


class Database(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger)

        # Schema rules used for validations
        self._schema = {
            "instance_name": {"type": "string", "maxlength": 100, "nullable": True},
            "database_id": {"type": "integer"},
            "asset_id": {"type": "integer"},
            "platform_id": {"type": "integer"},
            "port": {"type": "integer", "nullable": True},
            "version": {"type": "string", "maxlength": 20, "nullable": True},
        }
        self._validator = Validator(self._schema)

    def get_databases(self) -> list:
        """
        Returns all Databases.

        API: GET Databases

        Returns:
            list: List of Databases.
        """

        endpoint = "/databases"

        utils.print_log(
            self._logger,
            "Calling get_databases endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_database_by_id(self, database_id: int) -> dict:
        """
        Returns a Database by ID.

        API: GET Databases/{id}

        Args:
            database_id (int): The Database ID.

        Returns:
            dict: Database.
        """

        attributes = {"database_id": database_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/databases/{database_id}"

        utils.print_log(
            self._logger,
            "Calling get_database_by_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_databases_by_asset_id(self, asset_id: int) -> list:
        """
        Returns all Databases by Asset ID.

        API: GET Assets/{assetId}/Databases

        Args:
            asset_id (int): The Asset ID.

        Returns:
            list: List of Databases.
        """

        attributes = {"asset_id": asset_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/assets/{asset_id}/databases"

        utils.print_log(
            self._logger,
            "Calling get_databases_by_asset_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_database_by_asset_id(
        self,
        asset_id: int,
        platform_id: int,
        port: int,
        instance_name: str = None,
        is_default_instance: bool = None,
        version: str = None,
        template: str = None,
    ) -> dict:
        """
        Creates a new Database by Asset ID.

        API: POST Assets/{assetId}/Databases

        Args:
            asset_id (int): The Asset ID.
            platform_id (int): The Platform ID.
            port (int): The Port.
            instance_name (str, optional): The Instance Name.
            is_default_instance (bool, optional): Is Default Instance.
            version (str, optional): The Version.
            template (str, optional): The Template.

        Returns:
            dict: Database.
        """

        attributes = {
            "asset_id": asset_id,
            "platform_id": platform_id,
            "instance_name": instance_name,
            "port": port,
            "version": version,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            databases_fields, POST_DATABASES_ASSET_ID, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            platform_id=platform_id,
            instance_name=instance_name,
            is_default_instance=is_default_instance,
            port=port,
            version=version,
            template=template,
        )

        endpoint = f"/assets/{asset_id}/databases"

        utils.print_log(
            self._logger,
            "Calling post_database_by_asset_id endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def put_database_by_id(
        self,
        database_id: int,
        platform_id: int,
        instance_name: str = None,
        is_default_instance: bool = None,
        port: int = None,
        version: str = None,
        template: str = None,
    ) -> dict:
        """
        Updates a Database by ID.

        API: PUT Databases/{id}

        Args:
            database_id (int): The Database ID.
            platform_id (int): The Platform ID.
            instance_name (str, optional): The Instance Name.
            is_default_instance (bool, optional): Is Default Instance.
            port (int, optional): The Port.
            version (str, optional): The Version.
            template (str, optional): The Template.

        Returns:
            dict: Database.
        """

        attributes = {
            "database_id": database_id,
            "platform_id": platform_id,
            "instance_name": instance_name,
            "port": port,
            "version": version,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            databases_fields, PUT_DATABASES_ID, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            platform_id=platform_id,
            instance_name=instance_name,
            is_default_instance=is_default_instance,
            port=port,
            version=version,
            template=template,
        )

        endpoint = f"/databases/{database_id}"

        utils.print_log(
            self._logger,
            "Calling put_database_by_id endpoint",
            logging.DEBUG,
        )
        response = self._run_put_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def delete_database_by_id(self, database_id: int) -> None:
        """
        Deletes a Database by ID.

        API: DELETE Databases/{id}

        Args:
            database_id (int): The Database ID.

        Returns:
            None: If deletion is successful no exceptions.DeletionError is raised.
        """

        attributes = {"database_id": database_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/databases/{database_id}"

        utils.print_log(
            self._logger,
            "Calling delete_database_by_id endpoint",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)



"""
This file contains the fields mapping for each endpoint and version related to
Attributes.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-apis
- https://docs.beyondtrust.com/bips/docs/password-safe-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_ATTRIBUTE_ID,
    GET_ATTRIBUTES_ATTRIBUTE_TYPE_ID,
    GET_ATTRIBUTES_MANAGED_ACCOUNT_ID,
    GET_ATTRIBUTES_MANAGED_SYSTEM_ID,
    POST_ATTRIBUTE_ATTRIBUTE_TYPE_ID,
    POST_ATTRIBUTE_MANAGED_ACCOUNT_ID,
    POST_ATTRIBUTE_MANAGED_SYSTEM_ID,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ATTRIBUTES_ATTRIBUTE_TYPE_ID: {
        Version.DEFAULT: [
            "AttributeID",
            "AttributeTypeID",
            "ParentAttributeID",
            "ShortName",
            "LongName",
            "Description",
            "ValueInt",
            "IsReadOnly",
            "ChildAttributes",
        ],
    },
    GET_ATTRIBUTE_ID: {
        Version.DEFAULT: [
            "AttributeID",
            "AttributeTypeID",
            "ParentAttributeID",
            "ShortName",
            "LongName",
            "Description",
            "ValueInt",
            "IsReadOnly",
            "ChildAttributes",
        ]
    },
    POST_ATTRIBUTE_ATTRIBUTE_TYPE_ID: {
        Version.DEFAULT: {
            "ParentAttributeID": int,
            "ShortName": str,
            "LongName": str,
            "Description": str,
            "ValueInt": int,
        },
    },
    GET_ATTRIBUTES_MANAGED_ACCOUNT_ID: {
        Version.DEFAULT: [
            "AttributeID",
            "AttributeTypeID",
            "ParentAttributeID",
            "ShortName",
            "LongName",
            "Description",
            "IsReadOnly",
        ],
    },
    GET_ATTRIBUTES_MANAGED_SYSTEM_ID: {
        Version.DEFAULT: [
            "AttributeID",
            "AttributeTypeID",
            "ParentAttributeID",
            "ShortName",
            "LongName",
            "Description",
            "ValueInt",
            "IsReadOnly",
        ],
    },
    POST_ATTRIBUTE_MANAGED_ACCOUNT_ID: {
        Version.DEFAULT: {
            "AttributeID": int,
            "AttributeTypeID": int,
            "ParentAttributeID": int,
            "ShortName": str,
            "LongName": str,
            "Description": str,
            "IsReadOnly": bool,
        },
    },
    POST_ATTRIBUTE_MANAGED_SYSTEM_ID: {
        Version.DEFAULT: {
            "AttributeID": int,
            "AttributeTypeID": int,
            "ParentAttributeID": int,
            "ShortName": str,
            "LongName": str,
            "Description": str,
            "ValueInt": int,
            "IsReadOnly": bool,
        },
    },
}



"""
This file contains the fields mapping for each endpoint and version related to API
registrations.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_API_REGISTRATIONS,
    GET_API_REGISTRATIONS_ID,
    POST_API_REGISTRATIONS,
    PUT_API_REGISTRATIONS_ID,
)
from secrets_safe_library.constants.versions import Version

AUTHENTICATION_RULES = [
    {
        "Type": str,
        "Value": str,
        "Description": str,
    }
]

DEFAULT_GET_FIELDS = [
    "Id",
    "Name",
    "RegistrationType",
    "Active",
    "Visible",
    "AccessTokenDuration",
    "MultiFactorAuthenticationEnforced",
    "ClientCertificateRequired",
    "UserPasswordRequired",
    "VerifyPsrunSignature",
    "IPAuthenticationRules",
    "PSRUNRules",
    "XForwardedForAuthenticationRules",
]

# Used for POST and PUT requests for API Key and API Access policies
DEFAULT_BODY_API_REGISTRATIONS_API_KEY = {
    "Name": str,
    "RegistrationType": str,
    "AccessTokenDuration": int,
    "Active": bool,
    "Visible": bool,
    "MultiFactorAuthenticationEnforced": bool,
    "ClientCertificateRequired": bool,
    "UserPasswordRequired": bool,
    "VerifyPsrunSignature": bool,
    "IPAuthenticationRules": AUTHENTICATION_RULES,
    "PSRUNRules": [
        {
            "IPAddress": str,
            "MacAddress": str,
            "SystemName": str,
            "FQDN": str,
            "DomainName": str,
            "UserId": str,
            "RootVolumeId": str,
            "OSVersion": str,
        }
    ],
    "XForwardedForAuthenticationRules": AUTHENTICATION_RULES,
}

fields = {
    GET_API_REGISTRATIONS: {
        Version.DEFAULT: DEFAULT_GET_FIELDS,
    },
    GET_API_REGISTRATIONS_ID: {
        Version.DEFAULT: DEFAULT_GET_FIELDS,
    },
    POST_API_REGISTRATIONS: {
        Version.DEFAULT: DEFAULT_BODY_API_REGISTRATIONS_API_KEY,
    },
    PUT_API_REGISTRATIONS_ID: {
        Version.DEFAULT: DEFAULT_BODY_API_REGISTRATIONS_API_KEY,
    },
}



"""Permissions module, all the logic to manage safes from BI API"""

import logging

from cerberus import Validator

from secrets_safe_library import utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import ListMixin
from secrets_safe_library.utils import convert_as_literal


class Permission(APIObject, ListMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/permissions")

        # Schema rules used for validations
        self._schema = {
            "name": {"type": "string", "nullable": False},
        }
        self._validator = Validator(self._schema)

    def get_usergroup_permissions(self, usergroup_id: int) -> list:
        """
        Gets all permissions for the user group referenced by ID.

        API: GET UserGroups/{userGroupID}/Permissions

        Args:
            usergroup_id (int): The Usergroup ID.

        Returns:
            list: List of permissions.
        """

        endpoint = f"/usergroups/{usergroup_id}/permissions"

        utils.print_log(
            self._logger,
            "Calling get_usergroup_permissions endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def set_usergroup_permissions(
        self,
        usergroup_id: int,
        permissions: list,
    ) -> int:
        """
        Sets permissions for the user group referenced by ID.

        API: POST UserGroups/{userGroupId}/Permissions

        Args:
            usergroup_id (int): The Usergroup ID.
            permissions (list): List of permissions to set. Expected format:
                [
                    {
                        "PermissionID": int,
                        "AccessLevelID": int
                    },
                    ...
                ]

        Returns:
            int: Response status code.
        """

        permissions = convert_as_literal(permissions)

        endpoint = f"/usergroups/{usergroup_id}/permissions"

        utils.print_log(
            self._logger,
            "Calling post_usergroup_permissions endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint, permissions, include_api_version=False, expected_status_code=204
        )

        return response.status_code

    def delete_usergroup_permissions(self, usergroup_id: int) -> None:
        """
        Deletes all permissions for the user group referenced by ID.

        API: DELETE UserGroups/{userGroupId}/Permissions

        Args:
            usergroup_id (int): The Usergroup ID.

        Returns:
            None
        """

        endpoint = f"/usergroups/{usergroup_id}/permissions"

        utils.print_log(
            self._logger,
            "Calling delete_usergroup_permissions endpoint",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

        utils.print_log(
            self._logger,
            f"Permissions for user group with ID {usergroup_id} deleted successfully.",
            logging.INFO,
        )



"""
This file contains the fields mapping for each endpoint and version related to
Platforms.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-api
"""

from secrets_safe_library.constants.endpoints import GET_PLATFORMS
from secrets_safe_library.constants.versions import Version

fields = {
    GET_PLATFORMS: {
        Version.DEFAULT: [
            "PlatformID",
            "Name",
            "ShortName",
            "PortFlag",
            "DefaultPort",
            "SupportsElevationFlag",
            "DomainNameFlag",
            "AutoManagementFlag",
            "DSSAutoManagementFlag",
            "ManageableFlag",
            "DSSFlag",
            "LoginAccountFlag",
            "DefaultSessionType",
            "ApplicationHostFlag",
            "RequiresApplicationHost",
            "RequiresTenantID",
            "RequiresObjectID",
            "RequiresSecret",
        ],
    },
}



"""Users module, all the logic to manage users from BeyondInsight API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_USERS_AD,
    POST_USERS_APP,
    POST_USERS_BI,
    POST_USERS_LDAP,
    POST_USERS_USERGROUPID,
    PUT_USERS_ID_APP,
    PUT_USERS_ID_BI,
)
from secrets_safe_library.constants.users import UserType
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.users import fields as users_fields


class User(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger)

        # Schema rules used for validations
        self._schema = {
            "user_name": {"type": "string", "maxlength": 64, "nullable": True},
            "first_name": {"type": "string", "maxlength": 64, "nullable": True},
            "last_name": {"type": "string", "maxlength": 64, "nullable": True},
            "email_address": {"type": "string", "maxlength": 255, "nullable": True},
            "user_id": {"type": "integer", "nullable": True},
            "forest_name": {"type": "string", "maxlength": 300, "nullable": True},
            "domain_name": {"type": "string", "maxlength": 250, "nullable": True},
            "usergroup_id": {"type": "integer", "nullable": True},
            "distinguished_name": {
                "type": "string",
                "maxlength": 255,
                "nullable": True,
            },
        }
        self._validator = Validator(self._schema)

    def get_users(self, username: str = None, include_inactive: bool = False) -> list:
        """
        Returns a list of users.

        API: GET Users

        Args:
            username (str, optional): The user to return, in one of following formats:
                - username: returns the BeyondInsight users.
                - domain\\username or universal principal name: returns Active Directory
                                                                or LDAP users.
            include_inactive (bool, optional): Include inactive users. Default=False.

        Returns:
            list: List of users.
        """

        params = {"username": username, "includeInactive": include_inactive}

        query_string = self.make_query_string(params)

        endpoint = f"/users?{query_string}"

        utils.print_log(
            self._logger,
            "Calling get_users endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_user_by_id(self, user_id: int) -> dict:
        """
        Returns a user by ID.

        API: GET Users/{id}

        Args:
            user_id (int): The user ID.

        Returns:
            dict: User.
        """

        attributes = {"user_id": user_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/users/{user_id}"

        utils.print_log(
            self._logger,
            "Calling get_user_by_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_users_by_usergroup_id(self, usergroup_id: int) -> list:
        """
        Returns a list of users by User Group ID.

        API: GET UserGroups/{usergroupid}/Users

        Args:
            usergroup_id (int): The User Group ID.

        Returns:
            list: List of users.
        """

        attributes = {"usergroup_id": usergroup_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/usergroups/{usergroup_id}/users"

        utils.print_log(
            self._logger,
            "Calling get_users_by_usergroup_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_user_beyondinsight(
        self,
        user_name: str,
        first_name: str,
        email_address: str,
        password: str,
        last_name: str = None,
    ) -> dict:
        """
        Creates a new user with UserType "BeyondInsight".

        API: POST Users

        Args:
            user_name (str): The user name.
            first_name (str): The user's first name.
            email_address (str): The user's email address.
            password (str): The user's password.
            last_name (str, optional): The user's last name. Defaults to None.

        Returns:
            dict: Created user.
        """

        attributes = {
            "user_name": user_name,
            "first_name": first_name,
            "last_name": last_name,
            "email_address": email_address,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            users_fields, POST_USERS_BI, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            user_type=UserType.USER_TYPE_BI.value,
            user_name=user_name,
            first_name=first_name,
            last_name=last_name,
            email_address=email_address,
            password=password,
        )

        endpoint = "/users"

        utils.print_log(
            self._logger,
            "Calling post_user_beyondinsight endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_user_active_directory(
        self,
        user_name: str,
        forest_name: str = None,
        domain_name: str = None,
        bind_user: str = None,
        bind_password: str = None,
        use_ssl: bool = False,
    ) -> dict:
        """
        Creates a new user with UserType "ActiveDirectory".

        API: POST Users

        Args:
            user_name (str): The user name.
            forest_name (str, optional): The forest name. Defaults to None.
            domain_name (str, optional): The domain name. Defaults to None.
            bind_user (str, optional): The bind user. Defaults to None.
            bind_password (str, optional): The bind password. Defaults to None.
            use_ssl (bool, optional): Use SSL. Defaults to False.

        Returns:
            dict: Created user.
        """

        attributes = {
            "user_name": user_name,
            "forest_name": forest_name,
            "domain_name": domain_name,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            users_fields, POST_USERS_AD, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            user_type=UserType.USER_TYPE_AD.value,
            user_name=user_name,
            forest_name=forest_name,
            domain_name=domain_name,
            bind_user=bind_user,
            bind_password=bind_password,
            use_ssl=use_ssl,
        )

        endpoint = "/users"

        utils.print_log(
            self._logger,
            "Calling post_user_active_directory endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_user_ldap(
        self,
        host_name: str,
        distinguished_name: str,
        account_name_attribute: str,
        bind_user: str = None,
        bind_password: str = None,
        port: int = None,
        use_ssl: bool = False,
    ) -> dict:
        """
        Creates a new user with UserType "LDAP".

        API: POST Users

        Args:
            host_name (str): The LDAP host name.
            distinguished_name (str): The LDAP distinguished name.
            account_name_attribute (str): The LDAP account name attribute.
            bind_user (str, optional): The bind user.
            bind_password (str, optional): The bind password.
            port (int, optional): The LDAP port.
            use_ssl (bool, optional): Use SSL.

        Returns:
            dict: Created user.
        """

        attributes = {
            "distinguished_name": distinguished_name,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            users_fields, POST_USERS_LDAP, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            user_type=UserType.USER_TYPE_LDAP.value,
            host_name=host_name,
            distinguished_name=distinguished_name,
            account_name_attribute=account_name_attribute,
            bind_user=bind_user,
            bind_password=bind_password,
            port=port,
            use_ssl=use_ssl,
        )

        endpoint = "/users"

        utils.print_log(
            self._logger,
            "Calling post_user_ldap endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_user_application(
        self,
        user_name: str,
        access_policy_id: int = None,
    ) -> dict:
        """
        Creates a new user with UserType "Application".

        API: POST Users

        Args:
            user_name (str): The user name.
            access_policy_id (int, optional): The access policy ID.

        Returns:
            dict: Created user.
        """

        attributes = {"user_name": user_name}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            users_fields, POST_USERS_APP, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            user_type=UserType.USER_TYPE_APP.value,
            user_name=user_name,
            access_policy_id=access_policy_id,
        )

        endpoint = "/users"

        utils.print_log(
            self._logger,
            "Calling post_user_application endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_user_quarantine(self, user_id: int) -> dict:
        """
        Quarantines a user by ID.

        API: POST Users/{user_id}/quarantine

        Args:
            user_id (int): The user ID.

        Returns:
            dict: Quarantined user.
        """

        endpoint = f"/users/{user_id}/quarantine"

        utils.print_log(
            self._logger,
            "Calling post_user_quarantine endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint, payload={}, include_api_version=False
        )

        return response.json()

    def post_user_usergroupid(
        self,
        user_group_id: int,
        user_name: str,
        first_name: str,
        email_address: str,
        password: str,
        last_name: str = None,
    ) -> dict:
        """
        Creates a user in a BeyondInsight-type user group.

        API: UserGroups/{userGroupId}/Users

        Args:
            user_group_id (int): ID of the user group.
            user_name (str): The user name.
            first_name (str): The user's first name.
            email_address (str): The user's email address.
            password (str): The user's password.
            last_name (str, optional): The user's last name. Defaults to None.

        Returns:
            dict: Created user.
        """

        attributes = {
            "user_name": user_name,
            "first_name": first_name,
            "last_name": last_name,
            "email_address": email_address,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            users_fields, POST_USERS_USERGROUPID, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            user_name=user_name,
            first_name=first_name,
            last_name=last_name,
            email_address=email_address,
            password=password,
        )

        endpoint = f"/usergroups/{user_group_id}/users"

        utils.print_log(
            self._logger,
            "Calling post_user_usergroupid endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_user_recycleclient_secret(self, user_id: int) -> dict:
        """
        Recycles the client secret for an application user.

        API: POST Users/{user_id}/recycleclientsecret

        Args:
            user_id (int): The user ID.

        Returns:
            dict: User with recycled client secret.
        """

        endpoint = f"/users/{user_id}/recycleclientsecret"

        utils.print_log(
            self._logger,
            "Calling post_user_recycleclient_secret endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint, payload={}, include_api_version=False
        )

        return response.json()

    def put_user_beyondinsight(
        self,
        user_id: int,
        user_name: str,
        first_name: str,
        email_address: str,
        password: str,
        last_name: str = None,
    ) -> dict:
        """
        Updates a user with UserType "BeyondInsight" referenced by ID.

        API: PUT Users/{user_id}

        Args:
            user_id (int): ID of the BeyondInsight user.
            user_name (str): The user name.
            first_name (str): The user's first name.
            email_address (str): The user's email address.
            password (str): The user's password.
            last_name (str, optional): The user's last name. Defaults to None.

        Returns:
            dict: Updated user.
        """
        attributes = {
            "user_name": user_name,
            "first_name": first_name,
            "last_name": last_name,
            "email_address": email_address,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            users_fields, PUT_USERS_ID_BI, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            user_name=user_name,
            first_name=first_name,
            last_name=last_name,
            email_address=email_address,
            password=password,
        )

        endpoint = f"/users/{user_id}"

        utils.print_log(
            self._logger,
            "Calling put_user_beyondinsight endpoint",
            logging.DEBUG,
        )

        response = self._run_put_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def put_user_application(
        self,
        user_id: int,
        user_name: str,
        access_policy_id: int = None,
    ) -> dict:
        """
        Updates a user with UserType "Application" referenced by ID.

        API: PUT Users/{user_id}

        Args:
            user_id (int): ID of the application user.
            user_name (str): The user name.
            access_policy_id (int, optional): The access policy ID.

        Returns:
            dict: Updated user.
        """

        attributes = {"user_name": user_name}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            users_fields, PUT_USERS_ID_APP, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            user_name=user_name,
            access_policy_id=access_policy_id,
        )

        endpoint = f"/users/{user_id}"

        utils.print_log(
            self._logger,
            "Calling put_user_application endpoint",
            logging.DEBUG,
        )
        response = self._run_put_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def delete_user(self, user_id: int) -> None:
        """
        Deletes a user by ID.

        API: DELETE Users/{user_id}

        Args:
            user_id (int): The user ID.
        """

        attributes = {"user_id": user_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/users/{user_id}"

        utils.print_log(
            self._logger,
            "Calling delete_user endpoint",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)
        utils.print_log(
            self._logger,
            f"User with ID {user_id} deleted successfully.",
            logging.INFO,
        )



"""
This file contains the fields mapping for each endpoint and version related to
organizations.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-api
"""

from secrets_safe_library.constants.endpoints import GET_ORGANIZATIONS
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ORGANIZATIONS: {
        Version.DEFAULT: [
            "OrganizationID",
            "Name",
            "IsActive",
        ],
    },
}



"""
This file contains the fields mapping for each endpoint and version related to Roles.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-apis
"""

from secrets_safe_library.constants.endpoints import GET_ROLES
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ROLES: {
        Version.DEFAULT: [
            "RoleID",
            "Name",
        ],
    },
}



from enum import Enum


class Version(Enum):
    """
    Available request body versions from PS API.
    """

    DEFAULT = "DEFAULT"  # Used for entities that doesn't have versioning.
    V3_0 = "3.0"
    V3_1 = "3.1"
    V3_2 = "3.2"
    V3_3 = "3.3"
    V3_4 = "3.4"
    V3_5 = "3.5"



"""API registrations module, all the logic to manage API registrations from BI API"""

import logging

from secrets_safe_library import utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import POST_API_REGISTRATIONS
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.api_registrations import (
    fields as api_registrations_fields,
)
from secrets_safe_library.mixins import DeleteByIdMixin, GetByIdMixin, ListMixin
from secrets_safe_library.validators.api_registrations import APIRegistrationValidator


class APIRegistration(
    APIObject, GetByIdMixin, DeleteByIdMixin, ListMixin, APIRegistrationValidator
):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        APIObject.__init__(self, authentication, logger, endpoint="/apiregistrations")
        APIRegistrationValidator.__init__(self)

    def get_key_by_id(self, api_registration_id: int) -> str:
        """
        Retrieves the API key for an API Key policy API Registration.

        Args:
            api_registration_id (int): The ID of the API Registration to retrieve the
                key for.

        Returns:
            str: The API key content.
        """
        _ = self.validate(
            {"api_registration_id": api_registration_id},
            operation="get_key",
            allow_unknown=False,
        )
        endpoint = f"{self.endpoint}/{api_registration_id}/key"

        utils.print_log(self._logger, "Calling get_key_by_id endpoint", logging.DEBUG)
        response = self._run_get_request(
            endpoint, include_api_version=False, expected_status_code=[200, 201]
        )

        return response.text

    def rotate_api_key(self, api_registration_id: int) -> str:
        """
        Rotates the API key for an API Key policy API Registration.

        Args:
            api_registration_id (int): The ID of the API Registration to rotate the key
                for.

        Returns:
            str: The new API key content.
        """
        _ = self.validate(
            {"api_registration_id": api_registration_id},
            operation="rotate_key",
            allow_unknown=False,
        )

        endpoint = f"{self.endpoint}/{api_registration_id}/rotate"

        utils.print_log(self._logger, "Calling rotate_api_key endpoint", logging.DEBUG)

        response = self._run_post_request(
            endpoint, payload={}, include_api_version=False
        )

        return response.text

    def create_api_registration(
        self,
        name: str,
        registration_type: str,
        access_token_duration: int = 60,
        active: bool = True,
        visible: bool = True,
        multi_factor_authentication_enforced: bool = False,
        client_certificate_required: bool = False,
        user_password_required: bool = False,
        verify_psrun_signature: bool = False,
        ip_authentication_rules: list = None,
        psrun_rules: list = None,
        x_forwarded_for_authentication_rules: list = None,
    ) -> dict:
        """
        Creates a new API registration.

        API: POST ApiRegistrations

        Args:
            name (str): The registration name.
            registration_type (str): The type of registration, "ApiKeyPolicy" or
                "ApiAccessPolicy".
            access_token_duration (int, optional): Duration of the access token in
                minutes. Defaults to 60. Used with "ApiAccessPolicy" type.
            active (bool, optional): Whether the registration is active. Defaults to
                True.
            visible (bool, optional): Whether the registration is visible. Defaults to
                True.
            multi_factor_authentication_enforced (bool, optional): Whether MFA is
                enforced. Defaults to False.
            client_certificate_required (bool, optional): Whether a client certificate
                is required. Defaults to False.
            user_password_required (bool, optional): Whether a user password is
                required. Defaults to False.
            verify_psrun_signature (bool, optional): Whether to verify PSRun signature.
                Defaults to False.
            ip_authentication_rules (list, optional): List of IP authentication rules.
                Defaults to None, should be in this format: [{"Type": str,
                "Value": str, "Description": str}].
            psrun_rules (list, optional): List of PSRun rules. Defaults to None. Should
                be in this format: [{"Id": int, "IPAddress": str, "MacAddress": str,
                "SystemName": str, "FQDN": str, "DomainName": str, "UserId": str,
                "RootVolumeId": str, "OSVersion": str}]. Used with "ApiKeyPolicy" type.
            x_forwarded_for_authentication_rules (list, optional): List of
                X-Forwarded-For authentication rules. Defaults to None. should be in
                this format: [{"Type": str, "Value": str, "Description": str}].

        Returns:
            dict: The created API registration.
        """
        attributes = {
            "name": name,
            "registration_type": registration_type,
            "access_token_duration": access_token_duration,
            "active": active,
            "visible": visible,
            "multi_factor_authentication_enforced": (
                multi_factor_authentication_enforced
            ),
            "client_certificate_required": client_certificate_required,
            "user_password_required": user_password_required,
            "verify_psrun_signature": verify_psrun_signature,
            "ip_authentication_rules": utils.convert_as_literal(
                ip_authentication_rules
            ),
            "psrun_rules": utils.convert_as_literal(psrun_rules),
            "x_forwarded_for_authentication_rules": utils.convert_as_literal(
                x_forwarded_for_authentication_rules
            ),
        }

        validated_data = self.validate(
            attributes, operation="create", allow_unknown=False
        )

        req_structure = self.get_request_body_version(
            api_registrations_fields, POST_API_REGISTRATIONS, Version.DEFAULT
        )
        req_body = self.generate_request_body(
            req_structure,
            **validated_data,
        )

        utils.print_log(
            self._logger,
            f"Calling create API registration endpoint: {self.endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            self.endpoint, req_body, include_api_version=False
        )

        return response.json()

    def update_api_registration(
        self,
        registration_id: int,
        name: str = None,
        registration_type: str = None,
        access_token_duration: int = 60,
        active: bool = True,
        visible: bool = True,
        multi_factor_authentication_enforced: bool = False,
        client_certificate_required: bool = False,
        user_password_required: bool = False,
        verify_psrun_signature: bool = False,
        ip_authentication_rules: list = None,
        psrun_rules: list = None,
        x_forwarded_for_authentication_rules: list = None,
    ) -> dict:
        """
        Update an existing API registration.

        API: PUT ApiRegistrations/{id}

        Args:
            registration_id (int): The ID of the API registration to update.
            name (str): The registration name.
            registration_type (str): The type of registration, "ApiKeyPolicy" or
                "ApiAccessPolicy".
            access_token_duration (int, optional): Duration of the access token in
                minutes. Defaults to 60. Used with "ApiAccessPolicy" type.
            active (bool, optional): Whether the registration is active. Defaults to
                True.
            visible (bool, optional): Whether the registration is visible. Defaults to
                True.
            multi_factor_authentication_enforced (bool, optional): Whether MFA is
                enforced. Defaults to False.
            client_certificate_required (bool, optional): Whether a client certificate
                is required. Defaults to False.
            user_password_required (bool, optional): Whether a user password is
                required. Defaults to False.
            verify_psrun_signature (bool, optional): Whether to verify PSRun signature.
                Defaults to False.
            ip_authentication_rules (list, optional): List of IP authentication rules.
                Defaults to None, should be in this format: [{"Type": str,
                "Value": str, "Description": str}].
            psrun_rules (list, optional): List of PSRun rules. Defaults to None. Should
                be in this format: [{"Id": int, "IPAddress": str, "MacAddress": str,
                "SystemName": str, "FQDN": str, "DomainName": str, "UserId": str,
                "RootVolumeId": str, "OSVersion": str}]. Used with "ApiKeyPolicy" type.
            x_forwarded_for_authentication_rules (list, optional): List of
                X-Forwarded-For authentication rules. Defaults to None. should be in
                this format: [{"Type": str, "Value": str, "Description": str}].

        Returns:
            dict: The updated API registration.
        """
        attributes = {
            "registration_id": registration_id,
            "name": name,
            "registration_type": registration_type,
            "access_token_duration": access_token_duration,
            "active": active,
            "visible": visible,
            "multi_factor_authentication_enforced": (
                multi_factor_authentication_enforced
            ),
            "client_certificate_required": client_certificate_required,
            "user_password_required": user_password_required,
            "verify_psrun_signature": verify_psrun_signature,
            "ip_authentication_rules": utils.convert_as_literal(
                ip_authentication_rules
            ),
            "psrun_rules": utils.convert_as_literal(psrun_rules),
            "x_forwarded_for_authentication_rules": utils.convert_as_literal(
                x_forwarded_for_authentication_rules
            ),
        }

        validated_data = self.validate(
            attributes, operation="update", allow_unknown=False
        )

        req_structure = self.get_request_body_version(
            api_registrations_fields, POST_API_REGISTRATIONS, Version.DEFAULT
        )
        req_body = self.generate_request_body(
            req_structure,
            **validated_data,
        )

        endpoint = f"{self.endpoint}/{registration_id}"

        utils.print_log(
            self._logger,
            "Calling update API registration endpoint",
            logging.DEBUG,
        )
        response = self._run_put_request(endpoint, req_body, include_api_version=False)

        return response.json()



"""EPM Policies module, all the logic to manage EPM Policies from BI API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_EPM_POLICIES_ID_EPMAPPLICATIONS_ADD,
)
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.epm_policies import fields as epm_fields


class EPMPolicies(APIObject):
    """Class to interact with EPM Policies in BI API."""

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/epmpolicies")

        # Schema rules used for validations
        self._schema = {
            "policy_id": {"type": "string", "nullable": False},
            "group_name": {"type": "string", "maxlength": 128, "nullable": False},
            "name": {"type": "string", "maxlength": 128, "nullable": False},
            "path": {"type": "string", "maxlength": 256, "nullable": False},
            "publisher": {"type": "string", "maxlength": 128, "nullable": False},
            "children_inherit_token": {"type": "boolean", "nullable": False},
        }
        self._validator = Validator(self._schema)

    def add_epm_application(
        self,
        *,
        policy_id: str,
        group_name: str,
        name: str,
        path: str,
        publisher: str,
        children_inherit_token: bool,
    ) -> None:
        """
        Edits a policy to add an application, and updates this policy in the
        BeyondInsight database. Touches the LastModifiedDate to indicate that a change
        is made. Updated policy is deployed to agents per the usual process in
        BeyondInsight.

        API: POST /epmpolicies/{id}/epmapplications/add

        Args:
            policy_id (str): The GUID of the policy to update.
            group_name (str): The name of the group to add the application to.
            name (str): The name of the application.
            path (str): The path to the application.
            publisher (str): The publisher of the application.
            children_inherit_token (bool): Whether child applications inherit the token.

        Returns:
            None
        """

        attributes = {
            "policy_id": policy_id,
            "group_name": group_name,
            "name": name,
            "path": path,
            "publisher": publisher,
            "children_inherit_token": children_inherit_token,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"{self.endpoint}/{policy_id}/epmapplications/add"

        req_structure = self.get_request_body_version(
            epm_fields, POST_EPM_POLICIES_ID_EPMAPPLICATIONS_ADD, Version.DEFAULT
        )

        req_body = self.generate_request_body(req_structure, **attributes)

        utils.print_log(
            self._logger,
            "Calling add_epm_application endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint,
            payload=req_body,
            include_api_version=False,
            expected_status_code=200,
        )

        return response.json()



"""
This file contains the fields mapping for each endpoint and version related to Aliases.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_ALIASES,
    GET_ALIASES_ID,
    GET_ALIASES_NAME,
)
from secrets_safe_library.constants.versions import Version

DEFAULT_FIELDS = [
    "AliasID",
    "AliasName",
    "AliasState",
    "SystemId",
    "SystemName",
    "AccountId",
    "AccountName",
    "DomainName",
    "InstanceName",
    "DefaultReleaseDuration",
    "MaximumReleaseDuration",
    "LastChangeDate",
    "NextChangeDate",
    "IsChanging",
    "ChangeState",
    "MappedAccounts",
]

fields = {
    GET_ALIASES: {
        Version.DEFAULT: DEFAULT_FIELDS,
    },
    GET_ALIASES_ID: {
        Version.DEFAULT: DEFAULT_FIELDS,
    },
    GET_ALIASES_NAME: {
        Version.DEFAULT: DEFAULT_FIELDS,
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
Entity Types.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-api
"""

from secrets_safe_library.constants.endpoints import GET_ENTITY_TYPES
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ENTITY_TYPES: {
        Version.DEFAULT: [
            "EntityTypeID",
            "Name",
            "Description",
        ],
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
User group roles.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_USERGROUPS_ID_SMARTRULES_ROLES,
    POST_USERGROUPS_ID_SMARTRULES_ROLES,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_USERGROUPS_ID_SMARTRULES_ROLES: {
        Version.DEFAULT: ["RoleID", "Name"],
    },
    POST_USERGROUPS_ID_SMARTRULES_ROLES: {
        Version.DEFAULT: {"Roles": [{"RoleID": int}], "AccessPolicyID": int},
    },
}



"""Request Termination module, all the logic to manage requests from PasswordSafe API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_REQUEST_TERMINATION_MANAGED_ACCOUNT_ID,
    POST_REQUEST_TERMINATION_MANAGED_SYSTEM_ID,
    POST_REQUEST_TERMINATION_USER_ID,
)
from secrets_safe_library.core import APIObject


class RequestTermination(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger)

        # Schema rules used for validations
        self._schema = {
            "reason": {
                "type": "string",
                "maxlength": 1000,
                "nullable": True,
            },
        }
        self._validator = Validator(self._schema)

    def post_request_termination_managed_account_id(
        self,
        managed_account_id: int,
        reason: str = None,
    ) -> None:
        """
        Terminates a managed account request.

        API: POST Requests/{managedaccountid}/requests/terminate
        Args:
            managed_account_id (int): The managed account ID.
            reason (str, optional): The reason for the termination.
        """

        attibute = {
            "reason": reason,
        }
        if not self._validator.validate(attibute, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Reason": reason,
        }

        endpoint = "/" + POST_REQUEST_TERMINATION_MANAGED_ACCOUNT_ID.format(
            managedaccountid=managed_account_id
        ).replace("post:", "")

        utils.print_log(
            self._logger,
            "Calling post_request_termination_managed_account_id endpoint",
            logging.DEBUG,
        )
        self._run_post_request(
            endpoint, payload, include_api_version=False, expected_status_code=204
        )

    def post_request_termination_managed_system_id(
        self,
        managed_system_id: int,
        reason: str = None,
    ) -> None:
        """
        Terminates a managed system request.

        API: POST Requests/{managedsystemid}/requests/terminate
        Args:
            managed_system_id (int): The managed system ID.
            reason (str, optional): The reason for the termination.
        """

        attibute = {
            "reason": reason,
        }
        if not self._validator.validate(attibute, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Reason": reason,
        }

        endpoint = "/" + POST_REQUEST_TERMINATION_MANAGED_SYSTEM_ID.format(
            managedsystemid=managed_system_id
        ).replace("post:", "")

        utils.print_log(
            self._logger,
            "Calling post_request_termination_managed_system_id endpoint",
            logging.DEBUG,
        )
        self._run_post_request(
            endpoint, payload, include_api_version=False, expected_status_code=204
        )

    def post_request_termination_user_id(
        self,
        userid: int,
        reason: str = None,
    ) -> None:
        """
        Terminates a user request.

        API: POST Requests/{userid}/requests/terminate
        Args:
            userid (int): The user ID.
            reason (str, optional): The reason for the termination.
        """

        attibute = {
            "reason": reason,
        }
        if not self._validator.validate(attibute, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Reason": reason,
        }

        endpoint = "/" + POST_REQUEST_TERMINATION_USER_ID.format(userid=userid).replace(
            "post:", ""
        )

        utils.print_log(
            self._logger,
            "Calling post_request_termination_user_id endpoint",
            logging.DEBUG,
        )
        self._run_post_request(
            endpoint, payload, include_api_version=False, expected_status_code=204
        )



"""
Propagation Actions module, all the logic to manage Propagation Actions
from Password Safe API
"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_MANAGED_ACCOUNT_PROPAGATION_ACTIONS,
)
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.propagation_actions import (
    fields as propagation_actions_fields,
)
from secrets_safe_library.mixins import GetByIdMixin, ListMixin


class PropagationActions(APIObject, ListMixin, GetByIdMixin):
    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/propagationactions")

        # Schema rules used for validations
        self._schema = {
            "smart_rule_id": {"type": "integer", "nullable": True},
            "managed_account_id": {"type": "integer", "min": 1, "nullable": False},
            "propagation_action_id": {"type": "integer", "min": 1, "nullable": False},
        }

        self._validator = Validator(self._schema)

    def get_managed_account_propagation_actions(self, managed_account_id: int) -> list:
        """
        Retrieve a list of propagation actions for a specific managed account.

        API: GET /ManagedAccounts/{managed_account_id}/PropagationActions

        Args:
            managed_account_id (int): The ID of the managed account.

        Returns:
            list: A list of propagation actions for the specified managed account.
        """

        attributes = {"managed_account_id": managed_account_id}

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedaccounts/{managed_account_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            "Calling get_managed_account_propagation_actions endpoint",
            logging.DEBUG,
        )

        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_managed_account_propagation_action_by_id(
        self,
        managed_account_id: int,
        propagation_action_id: int,
        smart_rule_id: int = None,
    ) -> dict:
        """
        Assigns a propagation action to the managed account referenced by ID.

        API: POST
        /ManagedAccounts/{managed_account_id}/PropagationActions/{propagation_action_id}

        Args:
            managed_account_id (int): The ID of the managed account.
            propagation_action_id (int): The ID of the propagation action to initiate.
            smart_rule_id (int, optional): The ID of the smart rule to associate with
                                           the propagation action.

        Returns:
            dict: The response from the API after initiating the propagation action.
        """

        attributes = {
            "smart_rule_id": smart_rule_id,
            "managed_account_id": managed_account_id,
            "propagation_action_id": propagation_action_id,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            propagation_actions_fields,
            POST_MANAGED_ACCOUNT_PROPAGATION_ACTIONS,
            Version.DEFAULT,
        )

        req_body = self.generate_request_body(
            req_structure,
            smart_rule_id=smart_rule_id,
        )

        endpoint = (
            f"/managedaccounts/{managed_account_id}{self.endpoint}/"
            f"{propagation_action_id}"
        )

        utils.print_log(
            self._logger,
            "Calling post_managed_account_propagation_action_by_id endpoint",
            logging.DEBUG,
        )

        response = self._run_post_request(
            endpoint,
            req_body,
            include_api_version=False,
            expected_status_code=[200, 201],
        )

        return response.json()

    def delete_managed_account_propagation_action(
        self, managed_account_id: int
    ) -> None:
        """
        Unassigns all propagation actions from the managed account by ID.

        API: DELETE /ManagedAccounts/{managed_account_id}/PropagationActions

        Args:
            managed_account_id (int): The ID of the managed account.

        Returns:
            None
        """
        attributes = {
            "managed_account_id": managed_account_id,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedaccounts/{managed_account_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            "Calling delete_managed_account_propagation_action endpoint",
            logging.DEBUG,
        )

        self._run_delete_request(endpoint)

    def delete_managed_account_propagation_action_by_id(
        self, managed_account_id: int, propagation_action_id: int
    ) -> None:
        """
        Unassigns a propagation action from the managed account by ID.

        API: DELETE /ManagedAccounts/{managed_account_id}/PropagationActions/
             {propagation_action_id}

        Args:
            managed_account_id (int): The ID of the managed account.
            propagation_action_id (int): The ID of the propagation action to unassign.

        Returns:
            None
        """
        attributes = {
            "managed_account_id": managed_account_id,
            "propagation_action_id": propagation_action_id,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = (
            f"/managedaccounts/{managed_account_id}{self.endpoint}/"
            f"{propagation_action_id}"
        )

        utils.print_log(
            self._logger,
            "Calling delete_managed_account_propagation_action_by_id endpoint",
            logging.DEBUG,
        )

        self._run_delete_request(endpoint)



"""ManagedAccount Module, all the logic to retrieve managed accounts from PS API"""

import logging
from typing import Tuple

import requests

from secrets_safe_library import exceptions, utils
from secrets_safe_library.constants.endpoints import (
    POST_MANAGED_SYSTEMS_SYSTEM_ID_MANAGED_ACCOUNTS,
)
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.managed_accounts import (
    fields as managed_accounts_fields,
)
from secrets_safe_library.mixins import DeleteByIdMixin, GetByIdMixin
from secrets_safe_library.validators.managed_accounts import ManagedAccountValidator


class ManagedAccount(APIObject, DeleteByIdMixin, GetByIdMixin, ManagedAccountValidator):
    """
    Class to interact with managed accounts in PS API.
    """

    _separator = None
    _sign_app_out_error_message = "Error in sign_app_out"

    def __init__(self, authentication, logger=None, separator="/"):
        APIObject.__init__(self, authentication, logger, endpoint="/managedaccounts")

        ManagedAccountValidator.__init__(self)

        if len(separator.strip()) != 1:
            raise exceptions.LookupError(f"Invalid separator: {separator}")
        self._separator = separator

    def get_secret(self, path):
        """
        Get Managed account by path
        Arguments:
            path
        Returns:
            Retrieved managed account string
        """

        utils.print_log(
            self._logger,
            "Running get_secret method in ManagedAccount class",
            logging.DEBUG,
        )
        managed_account_dict = self.managed_account_flow([path])
        return managed_account_dict[path]

    def get_secret_with_metadata(self, path):
        """
        Get Managed account with metadata by path
        Arguments:
            path
        Returns:
             Retrieved managed account in dict format
        """

        utils.print_log(
            self._logger,
            "Running get_secret method in ManagedAccount class",
            logging.DEBUG,
        )
        managed_account_dict = self.managed_account_flow([path], get_metadata=True)
        return managed_account_dict

    def get_secrets(self, paths):
        """
        Get Managed accounts by paths
        Arguments:
            paths list
        Returns:
            Retrieved managed account in dict format
        """

        utils.print_log(
            self._logger,
            "Running get_secrets method in ManagedAccount class",
            logging.INFO,
        )
        managed_account_dict = self.managed_account_flow(paths)
        return managed_account_dict

    def get_secrets_with_metadata(self, paths):
        """
        Get Managed accounts with metadata by paths
        Arguments:
            paths list
        Returns:
            Retrieved managed account in dict format
        """

        utils.print_log(
            self._logger,
            "Running get_secrets method in ManagedAccount class",
            logging.INFO,
        )
        managed_account_dict = self.managed_account_flow(paths, get_metadata=True)
        return managed_account_dict

    def get_request_id(self, system_id, account_id):
        create_request_response = self.create_request(system_id, account_id)
        request_id = create_request_response.json()
        return request_id

    def managed_account_flow(self, paths, get_metadata=False):
        """
        Managed account by path flow
        Arguments:
            paths list
        Returns:
            Response (Dict)
        """

        response = {}

        for path in paths:

            utils.print_log(
                self._logger,
                f"************** managed account path len: {len(path)} **************",
                logging.INFO,
            )
            data = path.split(self._separator)

            if len(data) != 2:
                raise exceptions.LookupError(
                    f"Invalid managed account path: {path}. Use '{self._separator}' as "
                    f"a delimiter: system_name{self._separator}managed_account_name"
                )

            system_name = data[0]
            managed_account_name = data[1]

            manage_account = self.get_managed_accounts(
                system_name=system_name, account_name=managed_account_name
            )

            if get_metadata:
                response[f"{path}-metadata"] = manage_account

            utils.print_log(
                self._logger, "Managed account info retrieved", logging.DEBUG
            )

            request_id = self.get_request_id(
                manage_account["SystemId"], manage_account["AccountId"]
            )

            utils.print_log(
                self._logger,
                f"Request id retrieved: {'*' * len(str(request_id))}",
                logging.DEBUG,
            )

            if not request_id:
                raise exceptions.LookupError("Request Id not found")

            credential = self.get_credential_by_request_id(request_id)

            response[path] = credential

            utils.print_log(
                self._logger, "Credential was successfully retrieved", logging.INFO
            )

            self.request_check_in(request_id)
        return response

    def create_request(self, system_id: int, account_id: int) -> requests.Response:
        """
        Create request by system ID and account ID.

        API: POST Requests

        Args:
            system_id (int): ID of the managed system to request.
            account_id (int): ID of the managed account to request.

        Returns:
            request.Response: Response object.
        """
        payload = {
            "SystemID": system_id,
            "AccountID": account_id,
            "DurationMinutes": 5,
            "Reason": "Secrets Safe Integration",
            "ConflictOption": "reuse",
        }

        endpoint = "/Requests"
        utils.print_log(self._logger, "Calling create_request endpoint", logging.DEBUG)
        response = self._run_post_request(
            endpoint,
            payload=payload,
            expected_status_code=[200, 201],
            include_api_version=False,
        )
        return response

    def get_credential_by_request_id(self, request_id: int):
        """
        Retrieves the credentials for an approved and active (not expired) credentials
        release request.

        API: GET Credentials/{requestId}

        Args:
            request_id (int): The request ID.

        Returns:
            requests.Response: The response object containing the credential details.
        """

        endpoint = f"/Credentials/{request_id}"
        print_url = (
            f"{self._authentication._api_url}/Credentials/{'*' * len(str(request_id))}"
        )

        utils.print_log(
            self._logger,
            f"Calling get_credential_by_request_id endpoint: {print_url}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        credential = response.json()
        return credential

    def request_check_in(self, request_id: int, reason: str = "") -> None:
        """
        Checks-in/releases a request before it has expired.

        API: PUT Requests/{id}/checkin.

        Args:
            request_id (int): The ID of the request to check-in/release.
            reason (str, optional): A reason or comment why the request is being
                approved. Max string length is 1000.

        Returns:
            None
        """
        endpoint = f"/Requests/{request_id}/checkin"
        print_url = (
            f"{self._authentication._api_url}/Requests/"
            f"{'*' * len(str(request_id))}/checkin"
        )

        utils.print_log(
            self._logger,
            f"Calling request_check_in endpoint: {print_url}",
            logging.DEBUG,
        )
        payload = {"Reason": reason} if reason else {}
        _ = self._run_put_request(
            endpoint,
            payload=payload,
            expected_status_code=204,
        )

        utils.print_log(self._logger, "Request checked in", logging.DEBUG)

    def list_by_managed_system(self, managed_system_id: int) -> list:
        """
        Returns a list of managed accounts by managed system ID.

        API: GET ManagedSystems/{systemID}/ManagedAccounts

        Args:
            managed_system_id (int): Managed system ID.

        Returns:
            list: List of managed accounts.
        """

        endpoint = f"/managedsystems/{managed_system_id}/managedaccounts"

        utils.print_log(
            self._logger,
            "Calling list_by_managed_system endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)

        return response.json()

    def get_managed_accounts(
        self,
        *,
        system_name: str = None,
        account_name: str = None,
        system_id: str = None,
        workgroup_name: str = None,
        application_display_name: str = None,
        ip_address: str = None,
        type: str = None,
        limit: int = 1000,
        offset: int = 0,
    ) -> list | dict:
        """
        Get managed account(s) by system name, account name and other fields.

        API: GET ManagedAccounts.

        Args:
            system_name (str): The name of the system where the account is managed.
            account_name (str): The name of the account to retrieve.
            system_id (str): The ID of the managed system.
            workgroup_name (str): The name of the workgroup.
            application_display_name (str): The display name of the application.
            ip_address (str): The IP address of the managed asset.
            type (str): The type of the managed account to return. Options are:
                system, recent, domainlinked, database, cloud, application.
            limit (int): The number of records to return. Default is 1000.
            offset (int): The number of records to skip before returning records.
                Default is 0.

        Returns:
            list | dict: List of managed accounts if multiple accounts are found,
                otherwise, a dictionary containing a single accound data will be
                returned.

        Raises:
            exceptions.OptionsError: If provided arguments are not valid.
        """
        attributes = {
            "system_name": system_name,
            "account_name": account_name,
            "system_id": system_id,
            "workgroup_name": workgroup_name,
            "application_display_name": application_display_name,
            "ip_address": ip_address,
            "type": type,
            "limit": limit,
            "offset": offset,
        }

        params = {key.replace("_", ""): value for key, value in attributes.items()}

        self.validate(attributes, operation="list")

        query_string = self.make_query_string(params)
        endpoint = f"/managedaccounts?{query_string}"
        utils.print_log(
            self._logger,
            "Calling get_managed_accounts endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)
        return response.json()

    def list_by_smart_rule_id(self, smart_rule_id: int) -> list:
        """
        Returns a list of managed accounts by Smart Rule ID.

        API: GET SmartRules/{smartRuleID}/ManagedAccounts

        Args:
            smart_rule_id (int): Smart Rule ID.

        Returns:
            list: List of managed accounts.
        """

        endpoint = f"/smartrules/{smart_rule_id}/managedaccounts"

        utils.print_log(
            self._logger,
            "Calling list_by_smart_rule_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)

        return response.json()

    def list_by_quick_rule_id(self, quick_rule_id: int) -> list:
        """
        Returns a list of managed accounts by Quick Rule ID.

        API: GET QuickRules/{quickRuleID}/ManagedAccounts

        Args:
            quick_rule_id (int): Quick rule ID.

        Returns:
            list: List of managed accounts.
        """

        endpoint = f"/quickrules/{quick_rule_id}/managedaccounts"

        utils.print_log(
            self._logger,
            "Calling list_by_quick_rule_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)

        return response.json()

    def create_managed_account(
        self,
        *,
        system_id: int,
        account_name: str,
        password: str = None,
        domain_name: str = None,
        user_principal_name: str = None,
        sam_account_name: str = None,
        distinguished_name: str = None,
        private_key: str = None,
        passphrase: str = None,
        password_fallback_flag: bool = False,
        login_account_flag: bool = None,
        description: str = None,
        password_rule_id: int = 0,
        api_enabled: bool = False,
        release_notification_email: str = None,
        change_services_flag: bool = False,
        restart_services_flag: bool = False,
        change_tasks_flag: bool = False,
        release_duration: int = 120,
        max_release_duration: int = 525600,
        isa_release_duration: int = 120,
        max_concurrent_requests: int = 1,
        auto_management_flag: bool = False,
        dss_auto_management_flag: bool = False,
        check_password_flag: bool = False,
        reset_password_on_mismatch_flag: bool = False,
        change_password_after_any_release_flag: bool = False,
        change_frequency_type: str = "first",
        change_frequency_days: int = None,
        change_time: str = "23:30",
        next_change_date: str = None,
        use_own_credentials: bool = None,
        change_iis_app_pool_flag: bool = None,
        restart_iis_app_pool_flag: bool = None,
        workgroup_id: int = None,
        change_windows_auto_logon_flag: bool = False,
        change_com_plus_flag: bool = False,
        change_dcom_flag: bool = False,
        change_scom_flag: bool = False,
        object_id: str = None,
    ) -> Tuple[dict, int]:
        """
        Creates a new managed account in the managed system referenced by ID.

        API: POST ManagedSystems/{systemID}/ManagedAccounts

        Args:
            system_id (int): ID of the managed system where the account will be created.
            account_name (str): Name of the managed account.
            password (str): Password for the managed account.
            domain_name (str): Domain name of the managed account.
            user_principal_name (str): User principal name of the managed account.
            sam_account_name (str): SAM account name of the managed account.
            distinguished_name (str): Distinguished name of the managed account.
            private_key (str): Private key for the managed account, if applicable.
            passphrase (str): Passphrase for the private key, if applicable.
            password_fallback_flag (bool): True if password fallback is enabled,
                otherwise False.
            login_account_flag (bool): True if this is a login account, otherwise False.
            description (str): Description of the managed account.
            password_rule_id (int): ID of the password rule to apply to this account.
            api_enabled (bool): True if API access is enabled for this account,
                otherwise False.
            release_notification_email (str): Email address for release notifications.
            change_services_flag (bool): True if services should be updated with new
                credentials, otherwise False.
            restart_services_flag (bool): True if services should be restarted after a
                password change, otherwise False.
            change_tasks_flag (bool): True if tasks should be updated with new
                credentials, otherwise False.
            release_duration (int): Default release duration in minutes.
            max_release_duration (int): Maximum release duration in minutes.
            isa_release_duration (int): Default ISA release duration in minutes.
            max_concurrent_requests (int): Maximum number of concurrent requests
                allowed for this account.
            auto_management_flag (bool): True if auto-management is enabled, otherwise
                False.
            dss_auto_management_flag (bool): True if DSS key auto-management is enabled,
                otherwise False.
            check_password_flag (bool): True to enable password testing, otherwise
                False.
            reset_password_on_mismatch_flag (bool): True to reset password on mismatch,
                otherwise False.
            change_password_after_any_release_flag (bool): True to change password
                after any release, otherwise False.
            change_frequency_type (str): Type of change frequency ('first', 'last',
                'xdays').
            change_frequency_days (int): Number of days for scheduled changes when
                using 'xdays'.
            change_time (str): Time of day for scheduled changes 24hr format ('HH:MM').
            next_change_date (str): Next change date in 'YYYY-MM-DD' format.
            use_own_credentials (bool): True if the account should use its own
                credentials, otherwise False.
            change_iis_app_pool_flag (bool): True if IIS app pool should be changed,
                otherwise False.
            restart_iis_app_pool_flag (bool): True if IIS app pool should be restarted,
                otherwise False.
            workgroup_id (int, optional): ID of the workgroup to which the account
                belongs.
            change_windows_auto_logon_flag (bool): True if Windows auto logon should be
                changed, otherwise False.
            change_com_plus_flag (bool): True if COM+ settings should be changed,
                otherwise False.
            change_dcom_flag (bool): True if DCOM settings should be changed, otherwise
                False.
            change_scom_flag (bool): True if SCOM settings should be changed, otherwise
                False.
            object_id (str): Object ID of the managed account, if applicable.

        Returns:
            Tuple[dict, int]: A tuple containing the created managed account data and
                the HTTP status code.

        Raises:
            exceptions.OptionsError: If the provided attributes do not pass validation.
        """

        attributes = {
            "system_id": system_id,
            "account_name": account_name,
            "password": password,
            "domain_name": domain_name,
            "user_principal_name": user_principal_name,
            "sam_account_name": sam_account_name,
            "distinguished_name": distinguished_name,
            "private_key": private_key,
            "passphrase": passphrase,
            "password_fallback_flag": password_fallback_flag,
            "login_account_flag": login_account_flag,
            "description": description,
            "password_rule_id": password_rule_id,
            "api_enabled": api_enabled,
            "release_notification_email": release_notification_email,
            "change_services_flag": change_services_flag,
            "restart_services_flag": restart_services_flag,
            "change_tasks_flag": change_tasks_flag,
            "release_duration": release_duration,
            "max_release_duration": max_release_duration,
            "isa_release_duration": isa_release_duration,
            "max_concurrent_requests": max_concurrent_requests,
            "auto_management_flag": auto_management_flag,
            "dss_auto_management_flag": dss_auto_management_flag,
            "check_password_flag": check_password_flag,
            "reset_password_on_mismatch_flag": reset_password_on_mismatch_flag,
            "change_password_after_any_release_flag": (
                change_password_after_any_release_flag
            ),
            "change_frequency_type": change_frequency_type,
            "change_frequency_days": change_frequency_days,
            "change_time": change_time,
            "next_change_date": next_change_date,
            "use_own_credentials": use_own_credentials,
            "change_iis_app_pool_flag": change_iis_app_pool_flag,
            "restart_iis_app_pool_flag": restart_iis_app_pool_flag,
            "workgroup_id": workgroup_id,
            "change_windows_auto_logon_flag": change_windows_auto_logon_flag,
            "change_com_plus_flag": change_com_plus_flag,
            "change_dcom_flag": change_dcom_flag,
            "change_scom_flag": change_scom_flag,
            "object_id": object_id,
        }

        validated_data = self.validate(
            attributes, operation="create", allow_unknown=False
        )

        endpoint = f"/managedsystems/{system_id}{self.endpoint}"

        req_structure = self.get_request_body_version(
            managed_accounts_fields, POST_MANAGED_SYSTEMS_SYSTEM_ID_MANAGED_ACCOUNTS
        )

        req_body = self.generate_request_body(req_structure, **validated_data)

        utils.print_log(
            self._logger,
            "Calling create_managed_account endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint,
            payload=req_body,
            expected_status_code=[201],
            include_api_version=True,
        )
        managed_account = response.json()
        utils.print_log(
            self._logger, "Managed account created successfully", logging.INFO
        )
        return managed_account, response.status_code

    def assign_attribute(self, managed_account_id: int, attribute_id: int) -> dict:
        """
        Assigns an attribute to a managed account.

        API: POST ManagedAccounts/{managedAccountID}/Attributes/{attributeID}

        Args:
            managed_account_id (int): ID of the managed account.
            attribute_id (int): ID of the attribute to assign.

        Returns:
            dict: Response from the API
        """

        endpoint = f"{self.endpoint}/{managed_account_id}/attributes/{attribute_id}"
        utils.print_log(
            self._logger,
            "Calling assign_attribute endpoint",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint, payload={}, include_api_version=False
        )
        return response.json()

    def delete_attribute(self, managed_account_id: int, attribute_id: int) -> None:
        """
        Deletes an attribute to a managed account.

        API: DELETE ManagedAccounts/{managedAccountID}/Attributes/{attributeID}

        Args:
            managed_account_id (int): ID of the managed account.
            attribute_id (int): ID of the attribute to assign.

        Returns:
            None.
        """

        endpoint = f"{self.endpoint}/{managed_account_id}/attributes/{attribute_id}"
        utils.print_log(
            self._logger,
            "Calling delete_attribute endpoint",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

    def delete_all_attributes(self, managed_account_id: int) -> dict:
        """
        Deletes all managed account attributes by managed account ID.

        API: DELETE ManagedAccounts/{managedAccountID}/Attributes

        Args:
            managed_account_id (int): ID of the managed account.

        Returns:
            None.
        """

        endpoint = f"{self.endpoint}/{managed_account_id}/attributes"
        utils.print_log(
            self._logger,
            "Calling delete_all_attributes endpoint",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)



"""Replay module, all the logic to manage replay sessions from PS API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import DeleteByIdMixin, GetByIdMixin


class Replay(APIObject, GetByIdMixin, DeleteByIdMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/pbsm/replay")

        # Schema rules used for validations
        self._schema = {
            "id": {"type": "string", "nullable": False, "empty": False},
            "record_key": {"type": "string", "nullable": False, "empty": False},
            "protocol": {"type": "string", "nullable": False, "empty": False},
            "headless": {"type": "boolean", "nullable": False},
            "speed": {"type": "integer", "nullable": True, "min": 1},
            "offset": {"type": "integer", "nullable": True, "min": 0},
            "next": {"type": "integer", "nullable": True, "min": 0},
        }
        self._validator = Validator(self._schema)

    def create_replay_session(
        self,
        session_id: str,
        record_key: str,
        protocol: str,
        headless: bool = True,
    ) -> dict:
        """
        Creates a new replay session for a specified session token.

        API: POST pbsm/replay

        Args:
            session_id (str): Session Token from query to <base>/Sessions endpoint.
            record_key (str): RecordKey from query to <base>/Sessions endpoint.
            protocol (str): When session Type is 0 this should be RDP or for type 1 SSH.
            headless (bool): Must be set to true. Defaults to True.

        Returns:
            dict: Response containing:
                - id (str): ReplayID for this replay session
                - token (str): ReplayID for this replay session
                - ticket (str): Ticket value used internally

        Raises:
            exceptions.OptionsError: If validation fails.
            exceptions.CreationError: If the request fails (403, 404, etc).
        """

        attributes = {
            "id": session_id,
            "record_key": record_key,
            "protocol": protocol,
            "headless": headless,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "id": session_id,
            "record_key": record_key,
            "protocol": protocol,
            "headless": headless,
        }

        utils.print_log(
            self._logger,
            f"Calling create replay session endpoint: {self.endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            self.endpoint,
            payload,
            include_api_version=False,
            expected_status_code=[200],
        )

        return response.json()

    def get_replay_session(
        self,
        replay_id: str,
        jpeg_scale: str | None = None,
        png_scale: str | None = None,
        screen: bool | None = None,
    ) -> dict:
        """
        Displays the replay session details.

        API: GET pbsm/replay/{replayId}

        Args:
            replay_id (str): ID of the replay session returned from POST pbsm/replay.
            jpeg_scale (str, optional): Requests a JPEG image scaled by the given scale.
            png_scale (str, optional): Requests a PNG image scaled by the given scale.
            screen (bool, optional): If True, requests a text representation
                of the current SSH session.

        Returns:
            dict: Response containing:
                - tstamp (int): Start time of the session in seconds
                - end (int): End time of the session in seconds
                - offset (int): Current offset of replay session in ms
                - next (int): Offset of next activity of replay session in ms
                - speed (int): Speed of replay session as a %
                - eof (bool): Set to true when the end of the replay has been reached
                - duration (int): Duration in ms of the replay session

        Raises:
            exceptions.LookupError: If the request fails (403, 404, etc).
        """

        endpoint = f"{self.endpoint}/{replay_id}"

        # Build query parameters
        query_params = {}
        if jpeg_scale is not None:
            query_params["jpeg"] = jpeg_scale
        if png_scale is not None:
            query_params["png"] = png_scale
        if screen:
            query_params["screen"] = "1"

        if query_params:
            query_string = self.make_query_string(query_params)
            endpoint = f"{endpoint}?{query_string}"

        utils.print_log(
            self._logger,
            "Calling get replay session endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(
            endpoint, include_api_version=False, expected_status_code=[200]
        )

        return response.json()

    def control_replay_session(
        self,
        replay_id: str,
        speed: int | None = None,
        offset: int | None = None,
        next_frame: int | None = None,
    ) -> dict:
        """
        Controls the replay session status.

        API: PUT pbsm/replay/{replayId}

        Args:
            replay_id (str): ID of the replay session returned from POST pbsm/replay.
            speed (int, optional): Sets the replay speed of this session as a %.
            offset (int, optional): Sets the offset of the replay cursor
                for this session in ms.
            next_frame (int, optional): Requests the next changed frame
                based on the given % change.

        Returns:
            dict: Response containing:
                - tstamp (int): Start time of the session in seconds
                - end (int): End time of the session in seconds
                - offset (int): Current offset of replay session in ms
                - next (int): Offset of next activity of replay session in ms
                - speed (int): Speed of replay session as a %
                - eof (bool): Set to true when the end of the replay has been reached
                - duration (int): Duration in ms of the replay session

        Raises:
            exceptions.OptionsError: If validation fails.
            exceptions.UpdateError: If the request fails (403, 404, etc).
        """

        # Build attributes for validation
        attributes = {}
        if speed is not None:
            attributes["speed"] = speed
        if offset is not None:
            attributes["offset"] = offset
        if next_frame is not None:
            attributes["next"] = next_frame

        if attributes and not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        # Build payload
        payload = {}
        if speed is not None:
            payload["speed"] = speed
        if offset is not None:
            payload["offset"] = offset
        if next_frame is not None:
            payload["next"] = next_frame

        endpoint = f"{self.endpoint}/{replay_id}"

        utils.print_log(
            self._logger,
            "Calling control replay session endpoint",
            logging.DEBUG,
        )
        response = self._run_put_request(
            endpoint, payload, include_api_version=False, expected_status_code=[200]
        )

        return response.json()



"""Keystrokes module, all the logic to manage Keystrokes from PS API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import POST_KEYSTROKES_SEARCH
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.keystrokes import fields as keystrokes_fields
from secrets_safe_library.mixins import GetByIdMixin


class Keystroke(APIObject, GetByIdMixin):
    """Class to interact with Keystrokes in PS API."""

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/keystrokes")

        # Schema rules used for validations
        self._schema = {
            "session_id": {"type": "integer", "nullable": True},
            "data": {"type": "string", "nullable": False},
            "type": {
                "type": "integer",
                "min": 0,
                "max": 5,
                "default": 0,
                "nullable": True,
            },
        }
        self._validator = Validator(self._schema)

    def get_keystrokes_by_session_id(self, session_id: int) -> list:
        """
        Returns a list of keystrokes for the given session ID.

        API: GET Sessions/{sessionId:int}/Keystrokes

        Args:
            session_id (int): The Session ID.

        Returns:
            list: List of keystrokes for the specified session_id.
        """

        attributes = {"session_id": session_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/sessions/{session_id}/keystrokes"

        utils.print_log(
            self._logger,
            "Calling get_keystrokes_by_session_id endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)

        return response.json()

    def search_keystrokes(self, data: str, type: int = 0) -> list:
        """
        Searches for keystrokes based on data and type criteria.

        API: POST Keystrokes/Search

        Args:
            data (str): Keyword(s) for which to search.
            type (int, optional): Type of keystrokes:
                0: All (default)
                1: StdIn
                2: StdOut
                4: Window Event
                5: User Event

        Returns:
            list: List of keystrokes matching the search criteria.
        """

        attributes = {
            "data": data,
            "type": type,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            keystrokes_fields, POST_KEYSTROKES_SEARCH, Version.DEFAULT
        )
        req_body = self.generate_request_body(req_structure, **attributes)

        endpoint = f"{self.endpoint}/search"

        utils.print_log(
            self._logger, "Calling search keystrokes endpoint", logging.DEBUG
        )
        response = self._run_post_request(
            endpoint,
            payload=req_body,
            include_api_version=False,
            expected_status_code=200,
        )

        return response.json()



"""
Propagation Action Types module, all the logic to manage Propagation Action Types
from Password Safe API
"""

import logging

from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import ListMixin


class PropagationActionTypes(APIObject, ListMixin):
    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/propagationactiontypes")



"""
This file contains the fields mapping for each endpoint and version related to Address
Groups.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_ADDRESS_GROUPS,
    GET_ADDRESS_GROUPS_ID,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ADDRESS_GROUPS: {
        Version.DEFAULT: ["AddressGroupID", "Name", "OrganizationID"],
    },
    GET_ADDRESS_GROUPS_ID: {
        Version.DEFAULT: [
            "AddressGroupID",
            "Name",
        ],
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
Users.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-api
- https://docs.beyondtrust.com/bips/docs/secrets-safe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_USERS,
    GET_USERS_ID,
    GET_USERS_USERGROUPID,
    POST_USERS_AD,
    POST_USERS_APP,
    POST_USERS_BI,
    POST_USERS_LDAP,
    POST_USERS_QUARANTINE,
    POST_USERS_USERGROUPID,
    PUT_USERS_ID_APP,
    PUT_USERS_ID_BI,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_USERS: {
        Version.DEFAULT: [
            "UserID",
            "UserName",
            "DomainName",
            "DistinguishedName",
            "FirstName",
            "LastName",
            "EmailAddress",
            "LastLoginDate",
            "LastLoginAuthenticationType",
            "LastLoginConfigurationName",
            "LastLoginSAMLIDPURL",
            "LastLoginSSOURL",
            "IsQuarantined",
            "IsActive",
            "ClientID",
            "ClientSecret",
            "AccessPolicyID",
            "UserType",
        ],
    },
    GET_USERS_ID: {
        Version.DEFAULT: [
            "UserID",
            "UserName",
            "DomainName",
            "DistinguishedName",
            "FirstName",
            "LastName",
            "EmailAddress",
            "LastLoginDate",
            "LastLoginAuthenticationType",
            "LastLoginConfigurationName",
            "LastLoginSAMLIDPURL",
            "LastLoginSSOURL",
            "IsQuarantined",
            "IsActive",
            "ClientID",
            "ClientSecret",
            "AccessPolicyID",
            "UserType",
        ],
    },
    GET_USERS_USERGROUPID: {
        Version.DEFAULT: [
            "UserID",
            "UserName",
            "DomainName",
            "DistinguishedName",
            "FirstName",
            "LastName",
            "EmailAddress",
            "LastLoginDate",
            "LastLoginAuthenticationType",
            "LastLoginConfigurationName",
            "LastLoginSAMLIDPURL",
            "LastLoginSSOURL",
            "IsQuarantined",
            "IsActive",
            "ClientID",
            "ClientSecret",
            "AccessPolicyID",
            "UserType",
        ],
    },
    POST_USERS_BI: {
        Version.DEFAULT: {
            "UserType": str,
            "UserName": str,
            "FirstName": str,
            "LastName": str,
            "EmailAddress": str,
            "Password": str,
        },
    },
    POST_USERS_AD: {
        Version.DEFAULT: {
            "UserType": str,
            "UserName": str,
            "ForestName": str,
            "DomainName": str,
            "BindUser": str,
            "BindPassword": str,
            "UseSSL": bool,
        },
    },
    POST_USERS_LDAP: {
        Version.DEFAULT: {
            "UserType": str,
            "HostName": str,
            "DistinguishedName": str,
            "AccountNameAttribute": str,
            "BindUser": str,
            "BindPassword": str,
            "Port": int,
            "UseSSL": bool,
        },
    },
    POST_USERS_APP: {
        Version.DEFAULT: {
            "UserType": str,
            "UserName": str,
            "AccessPolicyID": int,
        },
    },
    POST_USERS_QUARANTINE: {
        Version.DEFAULT: {
            "UserID": int,
            "UserName": str,
            "DomainName": str,
            "DistinguishedName": str,
            "FirstName": str,
            "LastName": str,
            "EmailAddress": str,
            "IsQuarantined": bool,
            "IsActive": bool,
        },
    },
    POST_USERS_USERGROUPID: {
        Version.DEFAULT: {
            "UserName": str,
            "FirstName": str,
            "LastName": str,
            "EmailAddress": str,
            "Password": str,
        },
    },
    PUT_USERS_ID_BI: {
        Version.DEFAULT: {
            "UserName": str,
            "FirstName": str,
            "LastName": str,
            "EmailAddress": str,
            "Password": str,
        },
    },
    PUT_USERS_ID_APP: {
        Version.DEFAULT: {
            "UserName": str,
            "AccessPolicyID": int,
        },
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
Attribute Types.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_ATTRIBUTE_TYPES,
    GET_ATTRIBUTE_TYPES_ID,
    POST_ATTRIBUTE_TYPES,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ATTRIBUTE_TYPES: {
        Version.DEFAULT: [
            "AttributeTypeID",
            "Name",
            "IsReadOnly",
        ]
    },
    GET_ATTRIBUTE_TYPES_ID: {
        Version.DEFAULT: [
            "AttributeTypeID",
            "Name",
            "IsReadOnly",
        ]
    },
    POST_ATTRIBUTE_TYPES: {
        Version.DEFAULT: [
            "Name",
        ]
    },
}



"""Authentication Module, all the logic to authenticate in PS API"""

import json
import logging
import tempfile
from urllib.parse import urlparse

import requests

from secrets_safe_library import exceptions, utils

TIMEOUT_CONNECTION_SECONDS = 30
TIMEOUT_REQUEST_SECONDS = 30


class Authentication:

    _api_url = None
    _client_id = None
    _client_secret = None
    _verify_ca = True
    _token = None
    _certificate = None
    _certificate_key = None
    _sig_app_in_url = None
    _api_token = None
    _req = None
    _logger = None
    _timeout_connection_seconds = None
    _timeout_request_seconds = None
    _api_key = None
    _api_version = None

    def __init__(
        self,
        req: requests.sessions.Session = None,
        timeout_connection: int = 0,
        timeout_request: int = 0,
        api_url: str = "",
        client_id: str | None = None,
        client_secret: str | None = None,
        certificate: str | None = None,
        certificate_key: str | None = None,
        verify_ca: bool = True,
        logger: logging.Logger | None = None,
        api_key: str | None = None,
        api_version: str | None = None,
    ):

        self.validate_input("api_url", api_url)

        attributes = {}

        if api_version:
            attributes.update({"api_version": api_version})

        if api_key:
            attributes.update({"api_url": api_url, "api_key": api_key})
            inputs = utils.validate_inputs(attributes)
        else:
            self.validate_input("client_id", client_id)
            self.validate_input("client_secret", client_secret)
            attributes.update(
                {
                    "client_id": client_id,
                    "client_secret": client_secret,
                    "api_url": api_url,
                }
            )
            inputs = utils.validate_inputs(attributes)
            self._client_id = inputs["client_id"]
            self._client_secret = inputs["client_secret"]

        self._api_version = api_version

        self._api_url = inputs["api_url"]
        self._verify_ca = verify_ca
        self._sig_app_in_url = f"{self._api_url}/Auth/SignAppIn/"
        self._certificate = certificate
        self._certificate_key = certificate_key
        self._logger = logger
        self._timeout_connection_seconds = (
            timeout_connection or TIMEOUT_CONNECTION_SECONDS
        )
        self._timeout_request_seconds = timeout_request or TIMEOUT_REQUEST_SECONDS
        self._api_key = api_key

        utils.print_log(
            self._logger,
            "How long to wait for the server to connect and send data before "
            f"giving up: connection timeout: {self._timeout_connection_seconds} seconds"
            f", request timeout {self._timeout_request_seconds} seconds",
            logging.DEBUG,
        )

        self._req = req

        if not self._verify_ca:
            utils.print_log(
                self._logger,
                "verify_ca=false is insecure, it instructs the caller to not verify "
                "the certificate authority when making API calls.",
                logging.WARNING,
            )
            self._req.verify = False

    def oauth(self):
        """
        Get API Token
        Arguments:
            Client Id
            Secret
        Returns:
            Token
        """

        endpoint_url = self._api_url + "/Auth/connect/token"
        header = {"Content-Type": "application/x-www-form-urlencoded"}
        auth_info = {
            "client_id": self._client_id,
            "client_secret": self._client_secret,
            "grant_type": "client_credentials",
        }

        response = self._req.post(
            endpoint_url,
            auth_info,
            header,
            verify=self._verify_ca,
            timeout=(self._timeout_connection_seconds, self._timeout_request_seconds),
        )

        return response

    def sign_app_in(self):
        """
        Sign in to Secret safe API
        Arguments:
        Returns:
            logged user
        """

        utils.print_log(
            self._logger, f"Calling sign_app_in endpoint: {self._api_url}", logging.INFO
        )
        return self.send_post_sign_app_in()

    def get_api_access(self):
        """
        Get API Access
        Arguments:
        Returns:
            Result of sign_app_in call
        """

        if self._api_key:
            return self.sign_app_in()

        oauth_response = self.oauth()

        if oauth_response.status_code != 200:
            raise exceptions.AuthenticationFailure(
                f"Error getting token, message: {oauth_response.text}, statuscode: "
                f"{oauth_response.status_code}"
            )

        token_object = json.loads(oauth_response.text)
        self._api_token = token_object["access_token"]
        return self.sign_app_in()

    def sign_app_out(self):
        """
        Sign out to Secret safe API
        Arguments:
        Returns:
            Status of the action
        """
        url = f"{self._api_url}/Auth/Signout"
        utils.print_log(
            self._logger, f"Calling sign_app_out endpoint: {url}", logging.DEBUG
        )

        # Connection : close - tells secrets safe to close the session.
        response = self._req.post(
            url,
            timeout=(self._timeout_connection_seconds, self._timeout_request_seconds),
        )
        if response.status_code == 200:
            return True

    def send_post_sign_app_in(self):
        """
        Send Post request to Sign app in service
        Arguments:
        Returns:
            Service URL
            Certificate
        """

        headers = {"Authorization": f"Bearer {self._api_token}"}

        if self._api_key:
            utils.print_log(self._logger, "Using API key Authentication", logging.DEBUG)
            headers = {"Authorization": f"PS-Auth key={self._api_key}"}

        if self._certificate and self._certificate_key:
            with tempfile.NamedTemporaryFile(
                delete=True, mode="w", suffix=".pem"
            ) as cert_file:
                cert_file.write(self._certificate)
                cert_file.flush()

                with tempfile.NamedTemporaryFile(
                    delete=True, mode="w", suffix=".pem"
                ) as key_file:
                    key_file.write(self._certificate_key)
                    key_file.flush()

                    response = self._req.post(
                        self._sig_app_in_url,
                        headers=headers,
                        cert=(cert_file.name, key_file.name),
                    )
        else:
            response = self._req.post(self._sig_app_in_url, headers=headers)

        if self._api_token:
            del self._api_token

        if self._api_key:
            del self._api_key

        del headers

        return response

    def validate_input(self, parameter_name, parameter_value):
        """
        Validate input
        Arguments:
            Parameter name
            Parameter Value
        Returns:
        """
        if not parameter_value:
            raise exceptions.OptionsError(f"{parameter_name} parameter is missing")

        if parameter_name.lower() == "api_url":
            try:
                parsed_url = urlparse(parameter_value)
                all([parsed_url.scheme, parsed_url.netloc])
            except Exception:
                raise exceptions.OptionsError(
                    f"Url {parameter_value} is not valid, please check format"
                )



"""Auth Manager, Used to authenticate with the BeyondTrust PS API."""

# flake8: noqa: E402
# -*- coding: utf-8 -*-
# (c) 2025 BeyondTrust Inc.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
try:
    from secrets_safe_library import authentication
except ImportError:
    pass


class AuthManager:
    """
    Auth Manager, Used to authenticate with the BeyondTrust PS API.
    """

    def __init__(self, session, params: dict, logger=None) -> None:
        """
        Initialize a new AuthManager instance.

        Args:
            session (Session): Request session to make API calls.
            params (dict):  Mapping with fields related to authentication.
            logger (str): Logger object
        """
        self.session = session
        self.params = params
        self.logger = logger

    def authenticate(self):
        """
        Authenticate with the BeyondTrust PS API.
        """

        auth_obj = authentication.Authentication(
            req=self.session,
            timeout_connection=30,
            timeout_request=30,
            api_url=self.params["api_url"],
            client_id=self.params["client_id"],
            client_secret=self.params["client_secret"],
            verify_ca=True,
            logger=self.logger,
            api_version=self.params["api_version"],
        )

        response = auth_obj.get_api_access()

        if response.status_code != 200:
            return None

        return auth_obj



"""
This file contains the fields mapping for each endpoint and version related to
MANAGED SYSTEMS.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-api
- https://docs.beyondtrust.com/bips/docs/secrets-safe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_MANAGED_SYSTEMS,
    GET_MANAGED_SYSTEMS_ASSETID,
    GET_MANAGED_SYSTEMS_DATABASEID,
    GET_MANAGED_SYSTEMS_FUNCTIONALACCOUNTID,
    GET_MANAGED_SYSTEMS_MANAGEDSYSTEMID,
    GET_MANAGED_SYSTEMS_WORKGROUPID,
    POST_MANAGED_SYSTEMS_ASSETID,
    POST_MANAGED_SYSTEMS_DATABASEID,
    POST_MANAGED_SYSTEMS_WORKGROUPID,
    PUT_MANAGED_SYSTEMS_MANAGEDSYSTEMID,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_MANAGED_SYSTEMS: {
        Version.DEFAULT: [
            "ManagedSystemID",
            "EntityTypeID",
            "AssetID",
            "DatabaseID",
            "DirectoryID",
            "CloudID",
            "WorkgroupID",
            "HostName",
            "DnsName",
            "IPAddress",
            "InstanceName",
            "IsDefaultInstance",
            "Template",
            "ForestName",
            "UseSSL",
            "OracleInternetDirectoryID",
            "OracleInternetDirectoryServiceName",
            "SystemName",
            "PlatformID",
            "NetBiosName",
            "Port",
            "Timeout",
            "Description",
            "ContactEmail",
            "PasswordRuleID",
            "DSSKeyRuleID",
            "ReleaseDuration",
            "MaxReleaseDuration",
            "ISAReleaseDuration",
            "AutoManagementFlag",
            "FunctionalAccountID",
            "LoginAccountID",
            "ElevationCommand",
            "SshKeyEnforcementMode",
            "CheckPasswordFlag",
            "ChangePasswordAfterAnyReleaseFlag",
            "ResetPasswordOnMismatchFlag",
            "ChangeFrequencyType",
            "ChangeFrequencyDays",
            "ChangeTime",
            "AccountNameFormat",
            "RemoteClientType",
            "ApplicationHostID",
            "IsApplicationHost",
            "AccessURL",
        ],
    },
    GET_MANAGED_SYSTEMS_MANAGEDSYSTEMID: {
        Version.DEFAULT: [
            "ManagedSystemID",
            "EntityTypeID",
            "AssetID",
            "DatabaseID",
            "DirectoryID",
            "CloudID",
            "WorkgroupID",
            "HostName",
            "DnsName",
            "IPAddress",
            "InstanceName",
            "IsDefaultInstance",
            "Template",
            "ForestName",
            "UseSSL",
            "OracleInternetDirectoryID",
            "OracleInternetDirectoryServiceName",
            "SystemName",
            "PlatformID",
            "NetBiosName",
            "Port",
            "Timeout",
            "Description",
            "ContactEmail",
            "PasswordRuleID",
            "DSSKeyRuleID",
            "ReleaseDuration",
            "MaxReleaseDuration",
            "ISAReleaseDuration",
            "AutoManagementFlag",
            "FunctionalAccountID",
            "LoginAccountID",
            "ElevationCommand",
            "SshKeyEnforcementMode",
            "CheckPasswordFlag",
            "ChangePasswordAfterAnyReleaseFlag",
            "ResetPasswordOnMismatchFlag",
            "ChangeFrequencyType",
            "ChangeFrequencyDays",
            "ChangeTime",
            "AccountNameFormat",
            "RemoteClientType",
            "ApplicationHostID",
            "IsApplicationHost",
            "AccessURL",
        ],
    },
    GET_MANAGED_SYSTEMS_ASSETID: {
        Version.DEFAULT: [
            "ManagedSystemID",
            "EntityTypeID",
            "AssetID",
            "DatabaseID",
            "DirectoryID",
            "CloudID",
            "WorkgroupID",
            "HostName",
            "DnsName",
            "IPAddress",
            "InstanceName",
            "IsDefaultInstance",
            "Template",
            "ForestName",
            "UseSSL",
            "OracleInternetDirectoryID",
            "OracleInternetDirectoryServiceName",
            "SystemName",
            "PlatformID",
            "NetBiosName",
            "Port",
            "Timeout",
            "Description",
            "ContactEmail",
            "PasswordRuleID",
            "DSSKeyRuleID",
            "ReleaseDuration",
            "MaxReleaseDuration",
            "ISAReleaseDuration",
            "AutoManagementFlag",
            "FunctionalAccountID",
            "LoginAccountID",
            "ElevationCommand",
            "SshKeyEnforcementMode",
            "CheckPasswordFlag",
            "ChangePasswordAfterAnyReleaseFlag",
            "ResetPasswordOnMismatchFlag",
            "ChangeFrequencyType",
            "ChangeFrequencyDays",
            "ChangeTime",
            "AccountNameFormat",
            "RemoteClientType",
            "ApplicationHostID",
            "IsApplicationHost",
            "AccessURL",
        ],
    },
    GET_MANAGED_SYSTEMS_DATABASEID: {
        Version.DEFAULT: [
            "ManagedSystemID",
            "EntityTypeID",
            "AssetID",
            "DatabaseID",
            "DirectoryID",
            "CloudID",
            "WorkgroupID",
            "HostName",
            "DnsName",
            "IPAddress",
            "InstanceName",
            "IsDefaultInstance",
            "Template",
            "ForestName",
            "UseSSL",
            "OracleInternetDirectoryID",
            "OracleInternetDirectoryServiceName",
            "SystemName",
            "PlatformID",
            "NetBiosName",
            "Port",
            "Timeout",
            "Description",
            "ContactEmail",
            "PasswordRuleID",
            "DSSKeyRuleID",
            "ReleaseDuration",
            "MaxReleaseDuration",
            "ISAReleaseDuration",
            "AutoManagementFlag",
            "FunctionalAccountID",
            "LoginAccountID",
            "ElevationCommand",
            "SshKeyEnforcementMode",
            "CheckPasswordFlag",
            "ChangePasswordAfterAnyReleaseFlag",
            "ResetPasswordOnMismatchFlag",
            "ChangeFrequencyType",
            "ChangeFrequencyDays",
            "ChangeTime",
            "AccountNameFormat",
            "RemoteClientType",
            "ApplicationHostID",
            "IsApplicationHost",
            "AccessURL",
        ],
    },
    GET_MANAGED_SYSTEMS_FUNCTIONALACCOUNTID: {
        Version.DEFAULT: [
            "ManagedSystemID",
            "EntityTypeID",
            "AssetID",
            "DatabaseID",
            "DirectoryID",
            "CloudID",
            "WorkgroupID",
            "HostName",
            "DnsName",
            "IPAddress",
            "InstanceName",
            "IsDefaultInstance",
            "Template",
            "ForestName",
            "UseSSL",
            "OracleInternetDirectoryID",
            "OracleInternetDirectoryServiceName",
            "SystemName",
            "PlatformID",
            "NetBiosName",
            "Port",
            "Timeout",
            "Description",
            "ContactEmail",
            "PasswordRuleID",
            "DSSKeyRuleID",
            "ReleaseDuration",
            "MaxReleaseDuration",
            "ISAReleaseDuration",
            "AutoManagementFlag",
            "FunctionalAccountID",
            "LoginAccountID",
            "ElevationCommand",
            "SshKeyEnforcementMode",
            "CheckPasswordFlag",
            "ChangePasswordAfterAnyReleaseFlag",
            "ResetPasswordOnMismatchFlag",
            "ChangeFrequencyType",
            "ChangeFrequencyDays",
            "ChangeTime",
            "AccountNameFormat",
            "RemoteClientType",
            "ApplicationHostID",
            "IsApplicationHost",
            "AccessURL",
        ],
    },
    GET_MANAGED_SYSTEMS_WORKGROUPID: {
        Version.DEFAULT: [
            "ManagedSystemID",
            "EntityTypeID",
            "AssetID",
            "DatabaseID",
            "DirectoryID",
            "CloudID",
            "WorkgroupID",
            "HostName",
            "DnsName",
            "IPAddress",
            "InstanceName",
            "IsDefaultInstance",
            "Template",
            "ForestName",
            "UseSSL",
            "OracleInternetDirectoryID",
            "OracleInternetDirectoryServiceName",
            "SystemName",
            "PlatformID",
            "NetBiosName",
            "Port",
            "Timeout",
            "Description",
            "ContactEmail",
            "PasswordRuleID",
            "DSSKeyRuleID",
            "ReleaseDuration",
            "MaxReleaseDuration",
            "ISAReleaseDuration",
            "AutoManagementFlag",
            "FunctionalAccountID",
            "LoginAccountID",
            "ElevationCommand",
            "SshKeyEnforcementMode",
            "CheckPasswordFlag",
            "ChangePasswordAfterAnyReleaseFlag",
            "ResetPasswordOnMismatchFlag",
            "ChangeFrequencyType",
            "ChangeFrequencyDays",
            "ChangeTime",
            "AccountNameFormat",
            "RemoteClientType",
            "ApplicationHostID",
            "IsApplicationHost",
            "AccessURL",
        ],
    },
    POST_MANAGED_SYSTEMS_ASSETID: {
        Version.V3_0.value: {
            "PlatformID": int,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
        },
        Version.V3_1.value: {
            "PlatformID": int,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "RemoteClientType": str,
        },
        Version.V3_2.value: {
            "PlatformID": int,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "RemoteClientType": str,
            "ApplicationHostID": int,
            "IsApplicationHost": bool,
        },
    },
    POST_MANAGED_SYSTEMS_DATABASEID: {
        Version.DEFAULT: {
            "ContactEmail": str,
            "Description": str,
            "Timeout": int,
            "PasswordRuleID": int,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
        },
    },
    POST_MANAGED_SYSTEMS_WORKGROUPID: {
        Version.V3_0.value: {
            "EntityTypeID": int,
            "HostName": str,
            "IPAddress": str,
            "DnsName": str,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Template": str,
            "ForestName": str,
            "UseSSL": bool,
            "PlatformID": int,
            "NetBiosName": str,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "AccessURL": str,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "AccountNameFormat": int,
            "OracleInternetDirectoryID": int,
            "OracleInternetDirectoryServiceName": str,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
        },
        Version.V3_1.value: {
            "EntityTypeID": int,
            "HostName": str,
            "IPAddress": str,
            "DnsName": str,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Template": str,
            "ForestName": str,
            "UseSSL": bool,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "RemoteClientType": str,
            "PlatformID": int,
            "NetBiosName": str,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "AccountNameFormat": int,
            "OracleInternetDirectoryID": int,
            "OracleInternetDirectoryServiceName": str,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "AccessURL": str,
        },
        Version.V3_2.value: {
            "EntityTypeID": int,
            "HostName": str,
            "IPAddress": str,
            "DnsName": str,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Template": str,
            "ForestName": str,
            "UseSSL": bool,
            "PlatformID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "AccountNameFormat": int,
            "OracleInternetDirectoryID": int,
            "OracleInternetDirectoryServiceName": str,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "ApplicationHostID": int,
            "NetBiosName": str,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "IsApplicationHost": bool,
            "RemoteClientType": str,
            "AccessURL": str,
        },
        Version.V3_3.value: {
            "EntityTypeID": int,
            "HostName": str,
            "IPAddress": str,
            "DnsName": str,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Template": str,
            "ForestName": str,
            "UseSSL": bool,
            "OracleInternetDirectoryID": int,
            "OracleInternetDirectoryServiceName": str,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "RemoteClientType": str,
            "ApplicationHostID": int,
            "PlatformID": int,
            "NetBiosName": str,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "AccountNameFormat": int,
            "IsApplicationHost": bool,
            "AccessURL": str,
        },
    },
    PUT_MANAGED_SYSTEMS_MANAGEDSYSTEMID: {
        Version.V3_0.value: {
            "WorkgroupID": int,
            "HostName": str,
            "IPAddress": str,
            "DnsName": str,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Template": str,
            "ForestName": str,
            "UseSSL": bool,
            "PlatformID": int,
            "NetBiosName": str,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "AccessURL": str,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
        },
        Version.V3_1.value: {
            "WorkgroupID": int,
            "HostName": str,
            "IPAddress": str,
            "DnsName": str,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Template": str,
            "ForestName": str,
            "UseSSL": bool,
            "PlatformID": int,
            "NetBiosName": str,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "ReleaseDuration": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "RemoteClientType": str,
            "AccessURL": str,
        },
        Version.V3_2.value: {
            "WorkgroupID": int,
            "HostName": str,
            "IPAddress": str,
            "DnsName": str,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Template": str,
            "ForestName": str,
            "UseSSL": bool,
            "PlatformID": int,
            "NetBiosName": str,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "ContactEmail": str,
            "Description": str,
            "Port": int,
            "LoginAccountID": int,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "RemoteClientType": str,
            "ApplicationHostID": int,
            "IsApplicationHost": bool,
            "AccessURL": str,
        },
        Version.V3_3.value: {
            "WorkgroupID": int,
            "HostName": str,
            "ForestName": str,
            "UseSSL": bool,
            "PlatformID": int,
            "NetBiosName": str,
            "ContactEmail": str,
            "IPAddress": str,
            "DnsName": str,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Template": str,
            "Description": str,
            "Port": int,
            "Timeout": int,
            "SshKeyEnforcementMode": int,
            "PasswordRuleID": int,
            "DSSKeyRuleID": int,
            "LoginAccountID": int,
            "ReleaseDuration": int,
            "MaxReleaseDuration": int,
            "ISAReleaseDuration": int,
            "AutoManagementFlag": bool,
            "FunctionalAccountID": int,
            "ElevationCommand": str,
            "CheckPasswordFlag": bool,
            "ChangePasswordAfterAnyReleaseFlag": bool,
            "ResetPasswordOnMismatchFlag": bool,
            "ChangeFrequencyType": str,
            "ChangeFrequencyDays": int,
            "ChangeTime": str,
            "RemoteClientType": str,
            "ApplicationHostID": int,
            "IsApplicationHost": bool,
            "AccessURL": str,
        },
    },
}



"""SecretsSafe Module, all the logic to retrieve secrets from PS API"""

import logging
from urllib.parse import urlencode

import requests

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_SECRETS_SAFE_FOLDERS_FOLDERID,
    POST_SECRETS_SAFE_FOLDERS_FOLDERID_FILE,
    POST_SECRETS_SAFE_FOLDERS_FOLDERID_TEXT,
    PUT_SECRETS_SAFE_SECRETS_SECRETID,
    PUT_SECRETS_SAFE_SECRETS_SECRETID_FILE,
    PUT_SECRETS_SAFE_SECRETS_SECRETID_TEXT,
)
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.secrets import fields as secrets_fields
from secrets_safe_library.validators.secrets import SecretsValidator


class SecretsSafe(APIObject, SecretsValidator):
    _separator: str = None

    def __init__(
        self,
        authentication: Authentication,
        logger: logging.Logger = None,
        separator="/",
    ):
        super().__init__(authentication, logger, endpoint="/secrets-safe/secrets")

        if len(separator.strip()) != 1:
            raise exceptions.LookupError(f"Invalid separator: {separator}")
        self._separator = separator

        # Initialize the SecretsValidator parent class
        SecretsValidator.__init__(self)

    def get_secret(self, path):
        """
        Get secret by path
        Arguments:
            path
        Returns:
            Retrieved secret string
        """

        utils.print_log(
            self._logger,
            "Running get_secret method in SecretsSafe class",
            logging.DEBUG,
        )
        secrets_dict = self.secrets_by_path_flow([path])
        return secrets_dict[path]

    def get_secret_with_metadata(self, path):
        """
        Get secret by path with metadata
        Arguments:
            path
        Returns:
           Retrieved secret in dict format
        """

        utils.print_log(
            self._logger,
            "Running get_secret method in SecretsSafe class",
            logging.DEBUG,
        )
        secrets_dict = self.secrets_by_path_flow([path], get_metadata=True)
        return secrets_dict

    def get_secrets(self, paths):
        """
        Get secrets by paths
        Arguments:
            paths list
        Returns:
            Retrieved secret in dict format
        """

        utils.print_log(
            self._logger,
            "Running get_secrets method in SecretsSafe class",
            logging.INFO,
        )
        secrets_dict = self.secrets_by_path_flow(paths)
        return secrets_dict

    def get_secrets_with_metadata(self, paths):
        """
        Get secrets by paths with metadata
        Arguments:
            paths list
        Returns:
            Retrieved secret in dict format
        """

        utils.print_log(
            self._logger,
            "Running get_secrets method in SecretsSafe class",
            logging.INFO,
        )
        secrets_dict = self.secrets_by_path_flow(paths, get_metadata=True)
        return secrets_dict

    def get_all_secrets_by_folder_path(self, folder_path):
        """
        Get all secrets by folder path
        Arguments:
            folder path
        Returns:
            Response (Dict)
        """

        response = {}
        secret_response = self.get_secret_by_path(
            folder_path, None, self._separator, send_title=False
        )
        for secret in secret_response.json():
            secret_path = f"{secret['FolderPath']}/{secret['Title']}"
            response[f"{secret_path}-metadata"] = secret
            if secret["SecretType"] == "File":
                response[secret_path] = self.get_file_secret_data(secret["Id"])
            else:
                response[secret_path] = secret["Password"]
        return response

    def get_file_secret_data(self, secret_id) -> str:
        """
        Gets secret file as an attachment based on secretId.

        API: GET Secrets-Safe/Secrets/{secretId:guid}/file/download

        Args:
            secret_id (str): The secret id (GUID).

        Returns:
            file secret content (str).
        """

        utils.print_log(self._logger, "Getting secret by file", logging.DEBUG)
        try:
            file_response = self._get_file_by_id_req(secret_id)
        except exceptions.LookupError as e:
            raise exceptions.LookupError(f"Error getting file by id: {e}")

        return file_response.text

    def secrets_by_path_flow(self, paths, get_metadata=False):
        """
        Secrets by path flow
        Arguments:
            paths list
        Returns:
            Response (Dict)
        """

        response = {}
        for path in paths:

            if not path:
                continue

            data = path.split(self._separator)

            if len(data) < 2:
                raise exceptions.LookupError(
                    f"Invalid secret path: {path}, check your path and title separator,"
                    f" separator must be: {self._separator}"
                )

            folder_path = data[:-1]
            title = data[-1]

            try:
                secret_response = self.get_secret_by_path(
                    self._separator.join(folder_path), title, self._separator
                )
            except exceptions.LookupError as e:
                raise exceptions.LookupError(f"Error getting secret by path: {e}")

            secret = secret_response.json()
            if secret:
                if get_metadata:
                    response[f"{path}-metadata"] = secret[0]

                if secret[0]["SecretType"] == "File":
                    response[path] = self.get_file_secret_data(secret[0]["Id"])
                else:
                    response[path] = secret[0]["Password"]

                utils.print_log(
                    self._logger, "A secret was successfully retrieved", logging.INFO
                )
            else:
                raise exceptions.LookupError(f"{path}, Secret was not found")

        return response

    def get_secret_by_path(
        self, path: str, title: str, separator: str, send_title: bool = True
    ) -> requests.Response:
        """
        Get secrets by path and title.
        Arguments:
            path (str): The path where the secret is stored.
            title (str): The title used to identify the secret.
            separator (str): The separator used in the secret storage format.
            send_title (bool, optional): Flag to determine if the title should be
                included in the request. Defaults to True.

        Returns:
            requests.Response: A response object containing the secret.

        Raises:
            exceptions.OptionsError: If any of the schema rules are not valid.
        """

        path_depth = path.count(separator) + 1
        attributes = {"path": path, "title": title, "path_depth": path_depth}

        # Use the new SecretsValidator instead of the old Cerberus validator
        self.validate(attributes, operation="get_secret_by_path")

        query_params = {"path": path, "separator": separator}

        if self._authentication._api_version:
            query_params["version"] = self._authentication._api_version

        if send_title:
            query_params["title"] = title

        params = urlencode(query_params)

        endpoint = f"{self.endpoint}?{params}"

        utils.print_log(
            self._logger,
            "Calling get_secret_by_path endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)
        return response

    def _get_file_by_id_req(self, secret_id):
        """
        Get a File secret by File id
        Arguments:
            secret id
        Returns:
            File secret text
        """

        endpoint = f"{self.endpoint}/{secret_id}/file/download"

        utils.print_log(
            self._logger,
            "Calling _get_file_by_id_req endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)
        return response

    def get_secret_by_id(self, secret_id: str) -> dict:
        """
        Find a secret by ID.

        API: GET Secrets-Safe/Secrets/{secretId:guid}

        Args:
            secret_id (str): The secret ID (GUID).

        Returns:
            dict: Secret object according requested API version.

        Raises:
            exceptions.LookupError: Raised when no secret is found using secret_id.
        """

        endpoint = f"{self.endpoint}/{secret_id}"

        utils.print_log(
            self._logger,
            "Calling get_secret_by_id endpoint",
            logging.DEBUG,
        )

        try:
            response = self._run_get_request(endpoint)
        except exceptions.LookupError as e:
            raise exceptions.LookupError(f"Error getting secret by id: {e}")

        return response.json()

    def list_secrets(
        self,
        path: str = None,
        separator: str = None,
        title: str = None,
        afterdate: str = None,
        limit: int = None,
        offset: int = None,
    ) -> list:
        """
        Returns a list of secrets with the option to filter the list using query
        parameters.

        API: GET Secrets-Safe/Secrets

        Args:
            path (str): the full path to the secret.
            separator (str): the separator used in the path above. Default is /.
            title (str): the full title of the secret.
            afterdate (str): filter by modified or created on, after, or equal to the
            given date. Must be in the following UTC format: yyyy-MM-ddTHH:mm:ssZ.
            limit (int): limit the results.
            offset (int): skip the first (offset) number of secrets.

        Returns:
            list: List of secrets matching specified arguments.
        """

        params = {
            "path": path,
            "separator": separator,
            "title": title,
            "afterdate": afterdate,
            "limit": limit,
            "offset": offset,
        }
        query_params = {k: v for k, v in params.items() if v is not None}
        query_string = urlencode(query_params)
        endpoint = f"{self.endpoint}?{query_string}"

        utils.print_log(self._logger, "Calling list_secrets endpoint", logging.DEBUG)

        try:
            response = self._run_get_request(endpoint)
        except exceptions.LookupError as e:
            raise exceptions.LookupError(f"Error getting secrets: {e}")

        return response.json()

    def list_secrets_by_folder_id(self, folder_id: str = None) -> list:
        """
        Returns a list of secrets with the option to filter the list using query
        parameters.

        API: GET Secrets-Safe/Folders/{folderId:guid}/secrets

        Args:
            folder_id (str): The folder GUID.

        Returns:
            list: List of secrets for the given folder_id.
        """

        params = {"folderId": folder_id}
        query_string = urlencode(params)
        endpoint = f"{self.endpoint}?{query_string}"

        utils.print_log(
            self._logger,
            "Calling list_secrets_by_folder_id endpoint",
            logging.DEBUG,
        )

        try:
            response = self._run_get_request(endpoint)
        except exceptions.LookupError as e:
            raise exceptions.LookupError(f"Error getting secrets by folder: {e}")

        return response.json()

    def create_secret(
        self,
        title: str,
        folder_id: str,
        description: str = None,
        username: str = None,  # Used for normal Secrets
        password: str = None,  # Used for normal Secrets
        text: str = None,  # Used for Text Secrets
        file_path: str = None,  # Used for File Secrets
        owner_id: int = None,
        owner_type: str = None,
        owners: list = None,
        password_rule_id: int = None,
        notes: str = None,
        urls: list = None,
    ) -> dict:
        """
        Creates a Secret (Normal Secret, Text o File) using provided parameters
        according configured version.

        API:
            - POST Secrets-Safe/Folders/{folderId:guid}/secrets
            - POST Secrets-Safe/Folders/{folderId:guid}/secrets/text
            - POST Secrets-Safe/Folders/{folderId:guid}/secrets/file

        Args:
            title (str): The full title of the secret.
            folder_id (str): The GUID of the folder where secrets will be created.
            description (str, optional): The description of the secret.
            username (str, optional): The username for normal Secrets.
            password (str, optional): The password for normal Secrets.
            text (str, optional): The text content for Text Secrets.
            file_path (str, optional): The path to the file for File Secrets.
            owner_id (int, optional): The ID of the owner.
            owner_type (str, optional): The type of the owner.
            owners (list, optional): A list of owners.
            password_rule_id (int, optional): The ID of the password rule.
            notes (str, optional): Notes about the secret.
            urls (list, optional): A list of URLs with this format
                {'Id': GUID, 'CredentialId': GUID, 'Url': string}.

        Returns:
            dict: Created secret object according requested API version.
        """

        attributes = {
            "title": title,
            "description": description,
            "notes": notes,
            "owner_id": owner_id,
            "owner_type": owner_type,
            "owners": owners,
            "urls": urls,
        }

        # Identify which request body is being used
        if username:
            attributes.update(
                {
                    "username": username,
                    "password": password,
                }
            )

            self.validate(
                attributes,
                operation="create_credential_secret",
                version=self._authentication._api_version,
            )

            partial_endpoint = "secrets"
            utils.print_log(
                self._logger,
                f"Creating a normal secret: {partial_endpoint}",
                logging.DEBUG,
            )
            req_structure = self.get_request_body_version(
                secrets_fields, POST_SECRETS_SAFE_FOLDERS_FOLDERID
            )
        elif text:
            attributes.update({"text": text})

            self.validate(
                attributes,
                operation="create_text_secret",
                version=self._authentication._api_version,
            )

            partial_endpoint = "secrets/text"
            utils.print_log(
                self._logger,
                f"Creating a text secret: {partial_endpoint}",
                logging.DEBUG,
            )
            req_structure = self.get_request_body_version(
                secrets_fields, POST_SECRETS_SAFE_FOLDERS_FOLDERID_TEXT
            )
        elif file_path:
            attributes.update({"file_path": file_path})

            self.validate(
                attributes,
                operation="create_file_secret",
                version=self._authentication._api_version,
            )

            partial_endpoint = "secrets/file"
            utils.print_log(
                self._logger,
                f"Creating a file secret: {partial_endpoint}",
                logging.DEBUG,
            )
            req_structure = self.get_request_body_version(
                secrets_fields, POST_SECRETS_SAFE_FOLDERS_FOLDERID_FILE
            )
        else:
            raise exceptions.IncompleteArgumentsError(
                "Either username, text or file_path is required"
            )

        req_body = self.generate_request_body(
            req_structure,
            title=title,
            description=description,
            username=username,
            password=password,
            text=text,
            file_path=file_path,
            owner_id=owner_id,
            owner_type=owner_type,
            owners=owners,
            password_rule_id=password_rule_id,
            notes=notes,
            urls=urls,
            folder_id=folder_id,
        )

        endpoint = f"/secrets-safe/folders/{folder_id}/{partial_endpoint}"

        if partial_endpoint == "secrets/file":
            utils.print_log(
                self._logger,
                "Calling _run_post_file_request",
                logging.DEBUG,
            )
            response = self._run_post_file_request(
                endpoint,
                req_body,
                file_path=file_path,
            )
        else:
            utils.print_log(
                self._logger,
                "Calling _run_post_request",
                logging.DEBUG,
            )
            response = self._run_post_request(endpoint, req_body)

        return response.json()

    def delete_secret_by_id(self, secret_id: str) -> None:
        """
        Delete a secret by ID.

        API: DELETE Secrets-Safe/Secrets/{secretId:guid}/

        Args:
            secret_id (str): The secret ID (GUID).

        Returns:
            None: If deletion is successful no exceptions.DeletionError is raised.
        """

        endpoint = f"{self.endpoint}/{secret_id}"

        utils.print_log(
            self._logger,
            "Calling delete_secret_by_id endpoint",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

    def update_secret(
        self,
        secret_id: str,
        folder_id: str = None,
        title: str = None,
        description: str = None,
        username: str = None,  # Used for normal Secrets
        password: str = None,  # Used for normal Secrets
        text: str = None,  # Used for Text Secrets
        file_path: str = None,  # Used for File Secrets
        owner_id: int = None,
        owner_type: str = None,
        owners: list = None,
        password_rule_id: int = None,
        notes: str = None,
        urls: list = None,
    ) -> dict | None:
        """
        Updates a Secret (Normal Secret, Text o File) using provided parameters
        according configured version.

        API:
            - PUT Secrets-Safe/Secrets/{secretId:guid}/
            - PUT Secrets-Safe/Secrets/{secretId:guid}/text
            - PUT Secrets-Safe/Secrets/{secretId:guid}/file

        Args:
            secret_id (str): The ID of the secret to update.
            folder_id (str): The GUID of the folder where secrets will be created.
            title (str, optional): The full title of the secret.
            description (str, optional): The description of the secret.
            username (str, optional): The username for normal Secrets.
            password (str, optional): The password for normal Secrets.
            text (str, optional): The text content for Text Secrets.
            file_path (str, optional): The path to the file for File Secrets.
            owner_id (int, optional): The ID of the owner.
            owner_type (str, optional): The type of the owner.
            owners (list, optional): A list of owners.
            password_rule_id (int, optional): The ID of the password rule.
            notes (str, optional): Notes about the secret.
            urls (list, optional): A list of URLs with this format
                {'Id': GUID, 'CredentialId': GUID, 'Url': string}.

        Returns:
            dict: Created secret object according requested API version.
        """

        attributes = {
            "title": title,
            "description": description,
            "notes": notes,
            "folder_id": folder_id,
            "owner_id": owner_id,
            "owner_type": owner_type,
            "owners": owners,
            "urls": urls,
        }

        # Identify which request body is being used and validate accordingly
        partial_endpoint = ""
        if username:
            attributes.update(
                {
                    "username": username,
                    "password": password,
                    "password_rule_id": password_rule_id,
                }
            )

            self.validate(
                attributes,
                operation="update_credential_secret",
                version=self._authentication._api_version,
            )

            utils.print_log(
                self._logger,
                f"Updating a normal secret: {partial_endpoint}",
                logging.DEBUG,
            )
            req_structure = self.get_request_body_version(
                secrets_fields, PUT_SECRETS_SAFE_SECRETS_SECRETID
            )
        elif text:
            attributes.update({"text": text})
            self.validate(
                attributes,
                operation="update_text_secret",
                version=self._authentication._api_version,
            )

            partial_endpoint = "text"
            utils.print_log(
                self._logger,
                f"Updating a text secret: {partial_endpoint}",
                logging.DEBUG,
            )
            req_structure = self.get_request_body_version(
                secrets_fields, PUT_SECRETS_SAFE_SECRETS_SECRETID_TEXT
            )
        elif file_path:
            self.validate(
                attributes,
                operation="update_file_secret",
                version=self._authentication._api_version,
            )

            partial_endpoint = "file"
            utils.print_log(
                self._logger,
                f"Updating a file secret: {partial_endpoint}",
                logging.DEBUG,
            )
            req_structure = self.get_request_body_version(
                secrets_fields, PUT_SECRETS_SAFE_SECRETS_SECRETID_FILE
            )
        else:
            raise exceptions.IncompleteArgumentsError(
                "Either username, text or file_path is required"
            )

        req_body = self.generate_request_body(
            req_structure,
            title=title,
            description=description,
            username=username,
            password=password,
            text=text,
            file_path=file_path,
            owner_id=owner_id,
            owner_type=owner_type,
            owners=owners,
            password_rule_id=password_rule_id,
            notes=notes,
            urls=urls,
            folder_id=folder_id,
        )

        endpoint = f"{self.endpoint}/{secret_id}/{partial_endpoint}"

        if file_path:
            utils.print_log(
                self._logger,
                "Calling _run_put_file_request",
                logging.DEBUG,
            )
            _ = self._run_put_file_request(
                endpoint,
                req_body,
                file_path=file_path,
            )
            # File PUT request returns a 204 No content, so no need to return anything
            return
        else:
            utils.print_log(
                self._logger,
                "Calling _run_put_request",
                logging.DEBUG,
            )
            response = self._run_put_request(endpoint, req_body)

        return response.json()



import logging
import os

from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject


def configure_logging() -> None:
    propagate_setting = os.getenv("URLLIB3_PROPAGATE", "False").lower() in [
        "true",
        "1",
        "t",
    ]
    logging.getLogger("urllib3").propagate = propagate_setting


class Configuration(APIObject):
    """
    Class to manage configuration related operations from PS API.
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/configuration")

    def get_version(self) -> dict:
        """
        Returns the current system version.

        API: GET configuration/version

        Returns:
            dict: Version information containing Version field.
        """
        endpoint = f"{self.endpoint}/version"

        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()
